/* 
 * 
 * (c) Copyright Ascensio System Limited 2010-2014
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * http://www.gnu.org/licenses/agpl.html 
 * 
 */

if (typeof ASC === "undefined") {
    ASC = {};
}
if (typeof ASC.CRM === "undefined") {
    ASC.CRM = function() { return {} };
}

ASC.CRM.SettingsPage = (function() {

    var _initSortableFields = function() {
        var view = jq.getURLParam("view"),
            entityType = view != null && view != ""
                        ? view
                        : "contact";

        jq("#customFieldList").sortable({
            cursor: "move",
            handle: '.sort_row_handle',
            items: 'li',
            start: function(event, ui) {
                jq("#customFieldActionMenu").hide();
                jq("#customFieldList .entity-menu.active").removeClass("active");
            },
            beforeStop: function(event, ui) {
                jq(ui.item[0]).attr("style", "");
            },
            update: function(event, ui) {
                var fieldID = [];

                jq("#customFieldList li").find("[id^=custom_field_]").each(function() {
                    fieldID.push(jq(this).attr("id").split('_')[2]);
                });

                Teamlab.reorderCrmCustomFields({}, entityType, fieldID, {});
            }
        });
    };

    var _initFieldsActionMenu = function() {
        if (jq("#customFieldActionMenu").length != 1) return;

        jq.dropdownToggle({
            dropdownID: "customFieldActionMenu",
            switcherSelector: "#customFieldList .entity-menu",
            addTop: -2,
            addLeft: 10,
            rightPos: true,
            beforeShowFunction: function (switcherObj, dropdownItem) {
                var fieldId = switcherObj.attr("fieldid");
                if (!fieldId) return;
                if (parseInt(switcherObj.attr("data-relativeitemscount")) == 0) {
                    jq("#customFieldActionMenu .deleteField").removeClass("display-none");
                } else {
                    jq("#customFieldActionMenu .deleteField").addClass("display-none");
                }
            },
            showFunction: function(switcherObj, dropdownItem) {
                jq("#customFieldList .entity-menu.active").removeClass("active");
                if (dropdownItem.is(":hidden")) {
                    switcherObj.addClass("active");
                    if (switcherObj.attr("fieldid") != dropdownItem.attr("fieldid")) {
                        dropdownItem.attr("fieldid", switcherObj.attr("fieldid"));
                    }
                }
            },
            hideFunction: function() {
                jq("#customFieldList .entity-menu.active").removeClass("active");
            }
        });
    };

    var _showAddFieldPanel = function() {
        jq('#manageField div.containerHeaderBlock td:first').text(ASC.CRM.SettingsPage.AddCustomFieldHeaderText);
        jq('#manageField .crm-actionButtonsBlock a.button.blue.middle').text(ASC.CRM.SettingsPage.AddCustomFieldButtonText);
        jq('#manageField .crm-actionProcessInfoBlock > span').text(ASC.CRM.SettingsPage.AddCustomFieldProcessText);
        jq('#manageField .crm-actionProcessInfoBlock > img').attr("alt", ASC.CRM.SettingsPage.AddCustomFieldProcessText);

        _resetManageFieldPanel();
        RemoveRequiredErrorClass(jq("#manageField dl input:first"));
        jq('#manageField .crm-actionButtonsBlock a.button.blue.middle').unbind('click').click(function() {
            _createField();
        });
        PopupKeyUpActionProvider.EnableEsc = false;
        StudioBlockUIManager.blockUI("#manageField", 400, 400, 0);
    };

    var _initOtherActionMenu = function() {
        ASC.CRM.Common.removeExportButtons();

        jq("#menuCreateNewTask").bind("click", function() { ASC.CRM.TaskActionView.showTaskPanel(0, "", 0, null, {}); });
    };

    var _initNumericFieldsLimitation = function() {
        var maxCountRows = ASC.CRM.SettingsPage.maxCustomFieldRows,
            maxCountCols = ASC.CRM.SettingsPage.maxCustomFieldCols,
            maxCountSize = ASC.CRM.SettingsPage.maxCustomFieldSize;

        jq.forceIntegerOnly("#text_field_size");
        jq("#text_field_size").focusout(function(e) {
            var fieldSize = jq.trim(jq("#text_field_size").val());
            if (fieldSize != "" && fieldSize * 1 > maxCountSize) {
                jq("#text_field_size").val(maxCountSize);
            }
        });

        jq.forceIntegerOnly("#textarea_field_rows");
        jq.forceIntegerOnly("#textarea_field_cols");

        jq("#textarea_field_rows").focusout(function(e) {
            var fieldSize = jq.trim(jq("#textarea_field_rows").val());
            if (fieldSize != "" && fieldSize * 1 > maxCountRows) {
                jq("#textarea_field_rows").val(maxCountRows);
            }
        });

        jq("#textarea_field_cols").focusout(function(e) {
            var fieldSize = jq.trim(jq("#textarea_field_cols").val());
            if (fieldSize != "" && fieldSize * 1 > maxCountCols) {
                jq("#textarea_field_cols").val(maxCountCols);
            }
        });
    };

    var _findIndexOfFieldByID = function(fieldID) {
        for (var i = 0, n = ASC.CRM.SettingsPage.customFieldList.length; i < n; i++) {
            if (ASC.CRM.SettingsPage.customFieldList[i].id == fieldID) {
                return i;
            }
        }
        return -1;
    };

    var _resetManageFieldPanel = function() {
        jq("#manageField dl input:first").val("");
        jq("#manageField dl dd:last ul li:not(:first)").remove();
        jq('#manageField select').prop('value', '0');
        jq("#manageField dl .field_mask").hide();
        jq("#manageField dl .text_field").show();
        jq("#manageField input, select, textarea").removeAttr("readonly").removeAttr("disabled").removeClass('disabled');
        jq("#addOptionButton.display-none").removeClass("display-none");
        jq("#manageField .select_options .deleteBtn.display-none").removeClass("display-none");
    };


    var _fieldFactory = function(field, count) {
        if (jQuery.trim(field.mask) == "") {
            field.maskObj = "";
        } else {
            field.maskObj = jq.evalJSON(field.mask);
        }

        field.relativeItemsCount = count;
        field.relativeItemsString = ASC.CRM.SettingsPage.getLinkContactStringByIndex(count, jq.getURLParam("type"), jq.getURLParam("view"));
    };

    var _validateDataField = function() {
        if (jq("#manageField dl input:first").val().trim() == "") {
            ShowRequiredError(jq("#manageField dl input:first"), true);
            return false;
        } else {
            RemoveRequiredErrorClass(jq("#manageField dl input:first"));
            return true;
        }
    };

    var _readFieldData = function(sortOrder) {
        var view = jq.getURLParam("view"),
            field = {
                label: jq("#manageField dl input:first").val().trim(),
                fieldType: parseInt(jq("#manageField dl select:first").val()),
                entityType: view != null && view != "" ? view : "contact",
                position: sortOrder
            };

        switch (field.fieldType) {
            case 0:
                field.mask = jq.toJSON({ "size": jq("#text_field_size").val() });
                break;
            case 1:
                field.mask = jq.toJSON({
                    "rows": jq("#textarea_field_rows").val(),
                    "cols": jq("#textarea_field_cols").val()
                });
                break;
            case 2:
                var selectedItem = new Array();
                jq("#manageField dd.select_options :input").each(function() {
                    var value = jq(this).val().trim();
                    if (value == "") { return; }
                    selectedItem.push(jq(this).val());
                });

                if (selectedItem.length == 0) {
                    alert(ASC.CRM.Resources.CRMJSResource.EmptyItemList);
                    return;
                }
                field.mask = jq.toJSON(selectedItem);
                break;
            default:
                field.mask = "";
                break;
        }
        return field;
    };

    var _createField = function() {
        if (!_validateDataField()) { return; }

        var $listCustomFields = jq("#customFieldList li"),
            field = _readFieldData($listCustomFields.length),
            index = 0;

        for (var i = 0, len = $listCustomFields.length; i < len; i++) {
            if (jq($listCustomFields[i]).find(".customFieldTitle").text().trim() == field.label) {
                index = i + 1;
                break;
            }
        }
        if (index == 0) {
            Teamlab.addCrmCustomField({}, field.entityType, field, {
                before: function(params) {
                    jq("#manageField .crm-actionButtonsBlock").hide();
                    jq("#manageField .crm-actionProcessInfoBlock").show();
                },
                after: function(params) {
                    jq("#manageField .crm-actionProcessInfoBlock").hide();
                    jq("#manageField .crm-actionButtonsBlock").show();
                },
                success: ASC.CRM.SettingsPage.CallbackMethods.add_customField
            });
        } else {
            ASC.CRM.Common.animateElement({
                element: jq($listCustomFields[index - 1]),
                afterScrollFunc: jq.unblockUI()
            });
        }
    };

    var _editField = function (liObj, oldField, indexOfField) {
        var $listCustomFields = jq("#customFieldList li"),
            index = 0,
            field = null;
        if (!_validateDataField()) { return; }

        if (oldField.relativeItemsCount != 0) {
            var tmpField = _readFieldData(indexOfField);
            field = oldField;
            field.label = tmpField.label;
            field.entityType = tmpField.entityType;
        } else {
            field = _readFieldData(indexOfField);
            field.id = oldField.id;
        }

   
        for (var i = 0, n = $listCustomFields.length; i < n; i++) {
            if (jq($listCustomFields[i]).find(".customFieldTitle").text().trim() == field.label && jq($listCustomFields[i]).not(jq(liObj)).length != 0) {
                index = i + 1;
                break;
            }
        }
        if (index == 0) {
            Teamlab.updateCrmCustomField({ liObj: liObj, indexOfField: indexOfField }, field.entityType, field.id, field, {
                before: function() {
                    jq("#manageField .crm-actionButtonsBlock").hide();
                    jq("#manageField .crm-actionProcessInfoBlock").show();
                },
                after: function() {
                    jq("#manageField .crm-actionProcessInfoBlock").hide();
                    jq("#manageField .crm-actionButtonsBlock").show();
                },
                success: ASC.CRM.SettingsPage.CallbackMethods.edit_customField
            });
        } else {
            ASC.CRM.Common.animateElement({
                element: jq($listCustomFields[index - 1]),
                afterScrollFunc: jq.unblockUI()
            });
        }
    };

    return {
        CallbackMethods: {
            add_customField: function(params, field) {
                _fieldFactory(field, field.relativeItemsCount);
                ASC.CRM.SettingsPage.customFieldList.push(field);

                var $itemHTML = jq.tmpl("customFieldSettingsRowTmpl", field);
                if (jq("#customFieldList li").length == 0) {
                    jq("#emptyCustomFieldContent").hide();
                    jq("#customFieldList").show();
                    jq("#createNewField.display-none").removeClass("display-none");
                }
                $itemHTML.appendTo("#customFieldList");

                ASC.CRM.Common.animateElement({
                    element: $itemHTML,
                    afterScrollFunc: jq.unblockUI()
                });
            },

            edit_customField: function(params, field) {
                _fieldFactory(field, field.relativeItemsCount);
                ASC.CRM.SettingsPage.customFieldList[params.indexOfField] = field;

                var $itemHTML = jq.tmpl("customFieldSettingsRowTmpl", field);

                params.liObj.hide();
                $itemHTML.insertAfter(params.liObj);
                params.liObj.remove();

                ASC.CRM.Common.animateElement({
                    element: $itemHTML,
                    afterScrollFunc: jq.unblockUI()
                });
            },

            delete_customField: function(params, field) {
                params.liObj.remove();

                var index = _findIndexOfFieldByID(field.id);
                if (index != -1) { ASC.CRM.SettingsPage.customFieldList.splice(index, 1); }

                if (jq("#customFieldList li").length == 0) {
                    jq("#customFieldList").hide();
                    jq("#createNewField:not(.display-none)").addClass("display-none");
                    jq("#emptyCustomFieldContent").show();
                }
            }
        },

        init: function(defaultCustomFieldSize, defaultCustomFieldRows, defaultCustomFieldCols, maxCustomFieldSize, maxCustomFieldRows, maxCustomFieldCols) {
            ASC.CRM.SettingsPage.defaultCustomFieldSize = defaultCustomFieldSize;
            ASC.CRM.SettingsPage.defaultCustomFieldRows = defaultCustomFieldRows;
            ASC.CRM.SettingsPage.defaultCustomFieldCols = defaultCustomFieldCols;
            ASC.CRM.SettingsPage.maxCustomFieldSize = maxCustomFieldSize;
            ASC.CRM.SettingsPage.maxCustomFieldRows = maxCustomFieldRows;
            ASC.CRM.SettingsPage.maxCustomFieldCols = maxCustomFieldCols;

            if (typeof (window.customFieldList) == "undefined" ||
                typeof (window.relativeItems) == "undefined" ||
                window.customFieldList.length != window.relativeItems.length) { return; }

            for (var i = 0, len = window.customFieldList.length; i < len; i++) {
                _fieldFactory(window.customFieldList[i], window.relativeItems[i]);
            }

            ASC.CRM.SettingsPage.customFieldList = window.customFieldList;

            //jq("#customFieldList").disableSelection();
            jq("#createNewField").bind("click", function() { _showAddFieldPanel(); });
            jq("#emptyCustomFieldContent .emptyScrBttnPnl > a").bind("click", function() { _showAddFieldPanel(); });

            if (ASC.CRM.SettingsPage.customFieldList.length != 0) {
                jq("#customFieldList").show();
                jq("#createNewField.display-none").removeClass("display-none");
                jq.tmpl("customFieldSettingsRowTmpl", ASC.CRM.SettingsPage.customFieldList).appendTo("#customFieldList");
            } else {
                jq("#createNewField:not(.display-none)").addClass("display-none");
                jq("#emptyCustomFieldContent").show();
            }

            _initSortableFields();
            _initFieldsActionMenu();
            _initOtherActionMenu();
            _initNumericFieldsLimitation();
        },

        initEmptyScreen: function (imgSrc, headerText, describeText, buttonText) {
            jq.tmpl("emptyScrTmpl",
                {
                    ID: "emptyCustomFieldContent",
                    ImgSrc: imgSrc,
                    Header: headerText,
                    Describe: describeText,
                    ButtonHTML: ['<a class="link dotline blue plus">', buttonText, '</a>'].join('')
                }).insertAfter("#customFieldList");
        },

        startExportData: function () {
            jq("#exportDataContent a.button.blue.middle").hide();
            jq("#exportDataContent p.header-base-small").show();

            AjaxPro.CommonSettingsView.StartExportData(function(res) {
                if (res.error != null) {
                    alert(res.error.Message);
                    return;
                }
                ASC.CRM.SettingsPage.checkExportStatus(true);
            });
        },

        changeDefaultCurrency: function(ui) {
            var item = jq(ui);
            AjaxPro.onLoading = function(b) {
                if (b) {
                    item.prop("disabled", true);
                    item.next().show();
                    item.next('span').hide();
                } else {
                    item.prop("disabled", false);
                    item.next().hide();
                    item.next().next().show();
                }
            };

            AjaxPro.CommonSettingsView.SaveChangeSettings(item.val(), function(res) {
                if (res.error != null) {
                    alert(res.error.Message);
                    return;
                }
            });
        },

        deleteField: function() {
            jq("#customFieldActionMenu").hide();
            jq("#customFieldList .entity-menu.active").removeClass("active");

            var fieldid = jq("#customFieldActionMenu").attr('fieldid');
            if (typeof (fieldid) === "undefined" || fieldid == "" ||
                jq("#fieldMenu_" + fieldid).length == 0 || parseInt(jq("#fieldMenu_" + fieldid).attr("data-relativeitemscount")) != 0)
            { return; }


            var $menu = jq("#fieldMenu_" + fieldid);
            if ($menu.length != 1) { return; }

            var liObj = jq($menu.parents('li').get(0)),
                view = jq.getURLParam("view"),
                entityType = view != null && view != ""
                            ? view
                            : "contact";

            Teamlab.removeCrmCustomField({ liObj: liObj }, entityType, fieldid, {
                before: function(params) {
                    params.liObj.find(".entity-menu").hide();
                    params.liObj.find("div.ajax_loader").show();
                },
                success: ASC.CRM.SettingsPage.CallbackMethods.delete_customField
            });
        },

        showEditFieldPanel: function() {
            jq("#customFieldActionMenu").hide();
            jq("#customFieldList .entity-menu.active").removeClass("active");
            var fieldid = jq("#customFieldActionMenu").attr("fieldid"),
                $menu = null,
                liObj = null,
                index = -1,
                field = null;

            if (typeof (fieldid) === "undefined" || fieldid == "") { return; }

            $menu = jq("#fieldMenu_" + fieldid);
            if ($menu.length != 1) { return; }

            liObj = jq($menu.parents('li:first'));

            index = _findIndexOfFieldByID(fieldid);
            if (index === -1) { return; }

            field = ASC.CRM.SettingsPage.customFieldList[index];
            _resetManageFieldPanel();
            if (field.relativeItemsCount != 0) {
                jq("#manageField input, select, textarea").not("#manageField dl input:first").prop("readonly", "readonly").prop("disabled", "disabled").addClass('disabled');
                if (field.fieldType == 2) {//select box
                    jq("#addOptionButton:not(.display-none)").addClass("display-none");
                    jq("#manageField .select_options .deleteBtn:not(.display-none)").addClass("display-none");
                }
            }

            jq('#manageField div.containerHeaderBlock td:first').text(ASC.CRM.Resources.CRMJSResource.EditSelectedCustomField.replace('{0}', field.label));
            jq('#manageField .crm-actionButtonsBlock a.button.blue.middle').text(ASC.CRM.Resources.CRMJSResource.SaveChanges);
            jq('#manageField .crm-actionProcessInfoBlock > span').text(ASC.CRM.SettingsPage.EditCustomFieldProcessText);
            jq('#manageField .crm-actionProcessInfoBlock > img').attr("alt", ASC.CRM.SettingsPage.EditCustomFieldProcessText);


            jq("#manageField dl input:first").val(field.label);
            jq("#manageField dl select:first").val(field.fieldType);
            jq("#manageField dl .field_mask").hide();

            switch (field.fieldType) {
                case 0:
                    jq("#text_field_size").val(field.maskObj.size);
                    jq("#manageField dl .text_field").show();
                    break;
                case 1:
                    jq("#textarea_field_rows").val(field.maskObj.rows);
                    jq("#textarea_field_cols").val(field.maskObj.cols);
                    jq("#manageField dl .textarea_field").show();
                    break;
                case 2:
                    for (var i = 0, n = field.maskObj.length; i < n; i++) {
                        ASC.CRM.SettingsPage.toSelectBox(jq("#addOptionButton"));
                        jq("#manageField dd.select_options ul li:last input").val(field.maskObj[i])
                    }
                    jq("#manageField dl .select_options").show();
                    break;
                default:
                    break;
            }

            RemoveRequiredErrorClass(jq("#manageField dl input:first"));
            jq('#manageField .crm-actionButtonsBlock a.button.blue.middle').unbind('click').click(function() {
                _editField(liObj, field, index);
            });
            PopupKeyUpActionProvider.EnableEsc = false;
            StudioBlockUIManager.blockUI("#manageField", 400, 400, 0);
        },

        selectTypeEvent: function(selectObj) {
            var idx = selectObj.selectedIndex,
                which = parseInt(selectObj.options[idx].value);
            jq("#manageField dl .field_mask").hide();

            switch (which) {
                case 0:
                    jq("#manageField dl .text_field").show();
                    break;
                case 1:
                    jq("#manageField dl .textarea_field").show();
                    break;
                case 2:
                    jq("#manageField dl .select_options").show();
                    break;
            }
        },

        toSelectBox: function(buttonObj) {
            var ulObj = jq(buttonObj).prev();
            ulObj.children(":first").clone().show().appendTo(ulObj).focus();
        },

        getLinkContactStringByIndex: function(count, typeUrl, view) {
            var relativeItemsString = "",
                oneElement = "",
                manyElements = "";
            if (typeUrl == null) { typeUrl = ""; }
            if (view == null) { view = ""; }

            switch (typeUrl.trim()) {
                case "deal_milestone":
                    oneElement = ASC.CRM.Resources.CRMJSResource.OneDeal;
                    manyElements = ASC.CRM.Resources.CRMJSResource.ManyDeals;
                    break;
                case "contact_stage":
                case "contact_type":
                    oneElement = ASC.CRM.Resources.CRMJSResource.OneContact;
                    manyElements = ASC.CRM.Resources.CRMJSResource.ManyContacts;
                    break;
                case "task_category":
                    oneElement = ASC.CRM.Resources.CRMJSResource.OneTask;
                    manyElements = ASC.CRM.Resources.CRMJSResource.ManyTasks;
                    break;
                case "history_category":
                    oneElement = ASC.CRM.Resources.CRMJSResource.OneEvent;
                    manyElements = ASC.CRM.Resources.CRMJSResource.ManyEvents;
                    break;
                case "custom_field":
                case "tag":
                    switch (view.trim()) {
                        case "opportunity":
                            oneElement = ASC.CRM.Resources.CRMJSResource.OneDeal;
                            manyElements = ASC.CRM.Resources.CRMJSResource.ManyDeals;
                            break;
                        case "case":
                            oneElement = ASC.CRM.Resources.CRMJSResource.OneCase;
                            manyElements = ASC.CRM.Resources.CRMJSResource.ManyCases;
                            break;
                        default:
                            oneElement = ASC.CRM.Resources.CRMJSResource.OneContact;
                            manyElements = ASC.CRM.Resources.CRMJSResource.ManyContacts;
                            break;
                    }
            }

            if (count == 0) {
                return ASC.CRM.SettingsPage.getNoContactString(typeUrl, view);
            }
            if (count == 1) {
                relativeItemsString = "1 " + oneElement;
            } else {
                relativeItemsString = count + " " + manyElements;
            }
            return relativeItemsString;
        },

        getNoContactString: function(typeUrl, view) {
            var noElements = "";
            if (typeUrl == null) { typeUrl = ""; }
            if (view == null) { view = ""; }

            switch (typeUrl.trim()) {
                case "deal_milestone":
                    noElements = ASC.CRM.Resources.CRMJSResource.NoDeals;
                    break;
                case "contact_stage":
                case "contact_type":
                    noElements = ASC.CRM.Resources.CRMJSResource.NoContacts;
                    break;
                case "task_category":
                    noElements = ASC.CRM.Resources.CRMJSResource.NoTasks;
                    break;
                case "history_category":
                    noElements = ASC.CRM.Resources.CRMJSResource.NoEvents;
                    break;
                case "custom_field":
                case "tag":
                    switch (view.trim()) {
                        case "opportunity":
                            noElements = ASC.CRM.Resources.CRMJSResource.NoDeals;
                            break;
                        case "case":
                            noElements = ASC.CRM.Resources.CRMJSResource.NoCases;
                            break;
                        case "company":
                            noElements = ASC.CRM.Resources.CRMJSResource.NoCompanies;
                            break;
                        case "people":
                            noElements = ASC.CRM.Resources.CRMJSResource.NoPersons;
                            break;
                        default:
                            noElements = ASC.CRM.Resources.CRMJSResource.NoContacts;
                            break;
                    }
            }
            return noElements;
        },

        toggleCollapceExpand: function(elem) {
            $Obj = jq(elem);
            var isCollapse = $Obj.hasClass("headerCollapse");
            if (isCollapse) {
                $Obj.parents('li').nextUntil('#customFieldList li.expand_collapce_element').show();
            } else {
                $Obj.parents('li').nextUntil('#customFieldList li.expand_collapce_element').hide();
            }
            $Obj.toggleClass('headerExpand');
            $Obj.toggleClass('headerCollapse');
        },

        initSMTPSettings: function(testMailBodyText) {
            ASC.CRM.Common.removeExportButtons();
            jq("#menuCreateNewTask").bind("click", function() { ASC.CRM.TaskActionView.showTaskPanel(0, "", 0, null, {}); });

            jq.tmpl("SMTPSettingsFormTemplate", null).appendTo("#SMTPSettingsPannel");
            jq("#SMTPSettingsPannel").on("change", "#cbxAuthentication", function() {
                ASC.CRM.SettingsPage.changeAuthentication();
            });

            if (jq.browser.mobile === false) {
                ASC.CRM.SettingsPage.checkExportStatus(true);
            }

            ASC.CRM.SettingsPage.testMailBodyText = ASC.CRM.Common.convertText(Encoder.htmlDecode(testMailBodyText));

            jq.forceIntegerOnly("#tbxPort");

            if (typeof (window.SMTPSettings.EnableSSL) != "undefined" && window.SMTPSettings.EnableSSL != null) {
                jq("#tbxHost").val(window.SMTPSettings.Host);
                jq("#tbxPort").val(window.SMTPSettings.Port);
                jq("#tbxHostLogin").val(window.SMTPSettings.HostLogin);
                jq("#tbxHostPassword").val(window.SMTPSettings.HostPassword);
                jq("#tbxSenderDisplayName").val(window.SMTPSettings.SenderDisplayName);
                jq("#tbxSenderEmailAddress").val(window.SMTPSettings.SenderEmailAddress);
                jq("#cbxEnableSSL").prop("checked", window.SMTPSettings.EnableSSL);
                if (window.SMTPSettings.RequiredHostAuthentication) {
                    jq("#cbxAuthentication").prop("checked", true);
                    jq("#tbxHostLogin").removeAttr("disabled");
                    jq("#tbxHostPassword").removeAttr("disabled");
                } else {
                    jq("#cbxAuthentication").prop("checked", false);
                    jq("#tbxHostLogin").prop("disabled", true);
                    jq("#tbxHostPassword").prop("disabled", true);
                }
                jq("#showSendTestMailPanelBtn").removeClass("disable");
            }

            jq("#smtpSettingsContent table input").bind("change keyup", function() {
                jq("#showSendTestMailPanelBtn").addClass("disable");
            });

            jq("#sendTestMailPanelBtn").bind("click", function() {
                ASC.CRM.SettingsPage.sendTestMailSMTP();
            });

            jq("#sendTestMailPanel input.testMailToField, #sendTestMailPanel textarea.testMailBodyField ").bind("keyup", function() {
                var toEmail = jq("#sendTestMailPanel input.testMailToField").val().trim(),
                    mailBody = jq("#sendTestMailPanel textarea.testMailBodyField").val().trim();
                if (toEmail == "" || mailBody == "") {
                    jq("#sendTestMailPanelBtn").addClass("disable");
                } else {
                    jq("#sendTestMailPanelBtn").removeClass("disable");
                }
            });
        },

        changeAuthentication: function() {
            if (jq("#cbxAuthentication").is(":checked")) {
                jq("#tbxHostLogin").removeAttr("disabled");
                jq("#tbxHostPassword").removeAttr("disabled");
            } else {
                jq("#tbxHostLogin").prop("disabled", true);
                jq("#tbxHostPassword").prop("disabled", true);
            }
        },

        saveSMTPSettings: function() {
            jq("#smtpSettingsContent div.errorBox").remove();
            jq("#smtpSettingsContent div.okBox").remove();

            var host = jq("#tbxHost").val().trim(),
                port = jq("#tbxPort").val().trim(),
                authentication = jq("#cbxAuthentication").is(":checked"),
                hostLogin = jq("#tbxHostLogin").val().trim(),
                hostPassword = jq("#tbxHostPassword").val().trim(),
                senderDisplayName = jq("#tbxSenderDisplayName").val().trim(),
                senderEmailAddress = jq("#tbxSenderEmailAddress").val().trim(),
                enableSSL = jq("#cbxEnableSSL").is(":checked"),

                isValid = true;

            if (authentication
                && (host == "" || port == "" || hostLogin == "" || hostPassword == "" || senderDisplayName == "" || senderEmailAddress == "")) {
                    isValid = false;
            }
            if (!authentication
                && (host == "" || port == "" || senderDisplayName == "" || senderEmailAddress == "")) {
                    isValid = false;
            }

            if (!isValid) {
                jq("#smtpSettingsContent").prepend(
                    jq("<div></div>").addClass("errorBox").text(ASC.CRM.Resources.CRMJSResource.EmptyFieldsOfSettings)
                );
                return;
            }

            AjaxPro.onLoading = function(b) { if (b) { } else { } };

            AjaxPro.CommonSettingsView.SaveSMTPSettings(host, port, authentication, hostLogin, hostPassword, senderDisplayName, senderEmailAddress, enableSSL, function(res) {
                if (res.error != null) {
                    jq("#smtpSettingsContent").prepend(
                        jq("<div></div>").addClass("errorBox").text(res.error.Message)
                    );
                    return;
                }
                jq("#smtpSettingsContent div.errorBox").remove();
                jq("#smtpSettingsContent").prepend(
                    jq("<div></div>").addClass("okBox").text(ASC.CRM.Resources.CRMJSResource.SettingsUpdated)
                );
                jq("#showSendTestMailPanelBtn").removeClass("disable");
                setTimeout(function() {
                    jq("#smtpSettingsContent div.okBox").remove();
                }, 3000);
            });
        },

        showSendTestMailPanel: function() {
            if (jq("#showSendTestMailPanelBtn").hasClass("disable")) {
                return;
            }
            jq("#sendTestMailPanel textarea.testMailBodyField").val(ASC.CRM.SettingsPage.testMailBodyText);
            var email = jq("#tbxSenderEmailAddress").val().trim();
            jq("#sendTestMailPanel .testMailFromLabel").text(email);

            var toEmail = jq("#sendTestMailPanel input.testMailToField").val().trim(),
                mailBody = jq("#sendTestMailPanel textarea.testMailBodyField").val().trim();
            if (toEmail == "" || mailBody == "") {
                jq("#sendTestMailPanelBtn").addClass("disable");
            } else {
                jq("#sendTestMailPanelBtn").removeClass("disable");
            }
            PopupKeyUpActionProvider.EnableEsc = false;
            StudioBlockUIManager.blockUI("#sendTestMailPanel", 600, 320, 0);
        },

        sendTestMailSMTP: function() {
            var fromEmail = jq("#tbxSenderEmailAddress").val().trim(),
                toEmail = jq("#sendTestMailPanel input.testMailToField").val().trim(),
                mailSubj = jq("#sendTestMailPanel input.testMailSubjectField").val().trim(),
                mailBody = jq("#sendTestMailPanel textarea.testMailBodyField").val().trim();

            if (fromEmail == "" || toEmail == "" || mailBody == "") {
                return;
            }
            jq("#sendTestMailPanel .crm-actionButtonsBlock").hide();
            jq("#sendTestMailPanel .crm-actionProcessInfoBlock").show();


            AjaxPro.onLoading = function(b) { if (b) { } else { } };

            AjaxPro.CommonSettingsView.SendTestMailSMTP(toEmail, mailSubj, mailBody, function (res) {
                PopupKeyUpActionProvider.EnableEsc = true;
                jq.unblockUI();
                setTimeout(function() {
                    jq("#sendTestMailPanel .crm-actionProcessInfoBlock").hide();
                    jq("#sendTestMailPanel .crm-actionButtonsBlock").show();
                }, 0);
            });
        },

        checkExportStatus: function(isFirstVisit) {
            AjaxPro.onLoading = function(b) { };
            if (isFirstVisit) {
                ASC.CRM.SettingsPage.closeExportProgressPanel();
            }

            AjaxPro.CommonSettingsView.GetStatus(function(res) {
                if (res.error != null) {
                    alert(res.error.Message);
                    return false;
                }
                if (res.value == null) {
                    ASC.CRM.SettingsPage.closeExportProgressPanel();
                    return false;
                }

                jq("#exportDataContent div.progress").css("width", parseInt(res.value.Percentage) + "%");
                jq("#exportDataContent div.percent").text(parseInt(res.value.Percentage) + "%");
                jq("#exportDataContent a.button.blue.middle").hide();
                jq("#exportDataContent div.progress-container").show();
                jq("#exportDataContent div.middle-button-container").show();
                jq("#exportDataContent p.header-base-small").show();
                jq("#exportDataContent #abortButton").show();
                jq("#exportDataContent #okButton").hide();

                if (res.value.Error != null && res.value.Error != "") {
                    ASC.CRM.SettingsPage.buildErrorList(res);
                } else {
                    if (res.value.IsCompleted) {
                        jq("#exportDataContent #exportLinkBox span").html(
                            jq("<a></a>").attr("href", res.value.Status).text("exportdata.zip")
                        );
                        jq("#exportDataContent p.header-base-small").hide();
                        jq("#exportDataContent #exportLinkBox").show();
                        jq("#exportDataContent #abortButton").hide();
                        jq("#exportDataContent #okButton").show();
                    } else {
                        setTimeout("ASC.CRM.SettingsPage.checkExportStatus(false)", 3000);
                    }
                }
            });
        },

        abortExport: function() {
            AjaxPro.onLoading = function(b) { };
            AjaxPro.CommonSettingsView.Cancel(function(res) {
                if (res.error != null) { alert(res.error.Message); return; }
                ASC.CRM.SettingsPage.closeExportProgressPanel();
            });
        },

        closeExportProgressPanel: function() {
            jq("#exportDataContent div.progress").css("width", "0%");
            jq("#exportDataContent div.percent").text("0%");
            jq("#exportDataContent #abortButton").show();
            jq("#exportDataContent #okButton").hide();
            jq("#exportDataContent div.progress-container").hide();
            jq("#exportDataContent div.middle-button-container").hide();
            jq("#exportDataContent a.button.blue.middle").show();
            jq("#exportDataContent #exportErrorBox").hide();
            jq("#exportDataContent #exportLinkBox").hide();
            jq("#exportDataContent p.header-base-small").hide();
            jq("#exportDataContent div.progressErrorBox").html("");
            jq("#exportDataContent #exportLinkBox span").html("");
        },

        buildErrorList: function(res) {
            var mess = "error";
            switch (typeof res.value.Error) {
                case "object":
                    mess = res.value.Error.Message + "<br/>";
                    break;
                case "string":
                    mess = res.value.Error;
                    break;
            }

            jq("#exportDataContent div.progressErrorBox").html(
                jq("<div></div>").addClass("red-text").html(mess)
            );
            jq("#exportDataContent #exportErrorBox").show();
        }
    };
})();


ASC.CRM.ListItemView = (function() {

    var _initSortableElements = function() {
        jq("#listView").sortable({
            cursor: "move",
            handle: '.sort_drag_handle',
            items: 'li',
            start: function(event, ui) {
                if (ASC.CRM.ListItemView.CurrentType === 2 || ASC.CRM.ListItemView.CurrentType === 3) {
                    jq(".iconsPanelSettings").hide();
                }
                if (ASC.CRM.ListItemView.CurrentType === 1) {
                    jq("#colorsPanel").hide();
                    jq("#popup_colorsPanel").hide();
                }
                jq("#listItemActionMenu").hide();
                jq("#listView .entity-menu.active").removeClass("active");
            },
            stop: function(event, ui) {
                if (ASC.CRM.ListItemView.CurrentType === 2 || ASC.CRM.ListItemView.CurrentType === 3) {
                    var $activeImg = jq(ui.item).find("label.currentIcon");
                    if ($activeImg.length != 0) {
                        $activeImg.removeClass('active').removeClass('hover');
                    }
                }
            },

            beforeStop: function(event, ui) {
                jq(ui.item[0]).attr("style", "");
            },
            update: function(event, ui) {
                var itemsName = [];

                jq("#listView li[id*='list_item_id_']").each(function() {
                    var index = _findIndexOfItemByID(parseInt(jq(this).attr("id").replace("list_item_id_", "")));
                    if (index === -1) { return; }
                    itemsName.push(ASC.CRM.ListItemView.itemList[index].title);
                });
                Teamlab.reorderCrmListItems({}, ASC.CRM.ListItemView.CurrentType, itemsName, {});
            }
        });
    };

    var _initElementsActionMenu = function() {
        if (jq("#listItemActionMenu").length != 1) return;

        jq.dropdownToggle({
            dropdownID: "listItemActionMenu",
            switcherSelector: "#listView .entity-menu",
            addTop: -2,
            addLeft: 10,
            rightPos: true,
            showFunction: function(switcherObj, dropdownItem) {
                jq("#listView .entity-menu.active").removeClass("active");
                if (dropdownItem.is(":hidden")) {
                    switcherObj.addClass("active");
                    if (switcherObj.attr("listitemid") != dropdownItem.attr("listitemid")) {
                        dropdownItem.attr("listitemid", switcherObj.attr("listitemid"));
                    }
                }
            },
            hideFunction: function() {
                jq("#listView .entity-menu.active").removeClass("active");
            }
        });
    };

    var _initOtherActionMenu = function() {
        ASC.CRM.Common.removeExportButtons();
        jq("#menuCreateNewTask").bind("click", function() { ASC.CRM.TaskActionView.showTaskPanel(0, "", 0, null, {}); });
    };

    var _findIndexOfItemByID = function(id) {
        for (var i = 0, n = ASC.CRM.ListItemView.itemList.length; i < n; i++) {
            if (ASC.CRM.ListItemView.itemList[i].id == id) {
                return i;
            }
        }
        return -1;
    };

    var _resetManageItemPanel = function() {
        jq("#manageItem input").val("");
        jq("#manageItem textarea").val("");

        if (ASC.CRM.ListItemView.CurrentType === 2 || ASC.CRM.ListItemView.CurrentType === 3) {
            var $iconsPanel = jq("#iconsPanel_" + ASC.CRM.ListItemView.CurrentType);
            if ($iconsPanel.length == 1) {
                var $first_icon = $iconsPanel.children('label:first');
                if ($first_icon.length == 1) {
                    var $selectedIcinObj = jq("#manageItem label.selectedIcon"),
                        cssClass = $first_icon.attr('img_name').split('.')[0];

                    $selectedIcinObj.attr('img_name', $first_icon.attr('img_name'));
                    $selectedIcinObj.attr('title', $first_icon.attr('title'));
                    $selectedIcinObj.attr('alt', $first_icon.attr('alt'));
                    $selectedIcinObj.attr("class", ["selectedIcon ", ASC.CRM.ListItemView.CurrentType == 2 ? "task_category" : "event_category", " ", cssClass].join(''));
                }
            }
        }
        if (ASC.CRM.ListItemView.CurrentType === 1) {
            var $divColors = jq("#colorsPanel > span"),
                ind = Math.floor(Math.random() * $divColors.length);
            jq("#manageItem .selectedColor").css("background-color", ASC.CRM.Common.getHexRGBColor(jq($divColors.get(ind)).css("background-color")));
        }
    };

    var _showAddItemPanel = function() {
        jq('#manageItem div.containerHeaderBlock td:first').text(ASC.CRM.ListItemView.AddItemHeaderText);
        jq('#manageItem .crm-actionButtonsBlock a.button.blue.middle').text(ASC.CRM.ListItemView.AddItemButtonText);

        jq('#manageItem .crm-actionProcessInfoBlock > span').text(ASC.CRM.ListItemView.AddItemProcessText);
        jq('#manageItem .crm-actionProcessInfoBlock > img').attr("alt", ASC.CRM.ListItemView.AddItemProcessText);

        _resetManageItemPanel();

        jq('#manageItem .crm-actionButtonsBlock a.button.blue.middle').unbind('click').click(function() {
            _createItem();
        });
        RemoveRequiredErrorClass(jq("#manageItem input:first"));
        PopupKeyUpActionProvider.CloseDialogAction = "javascript:jq('.iconsPanelSettings').hide();jq('#colorsPanel').hide();jq('#popup_colorsPanel').hide();";
        PopupKeyUpActionProvider.EnableEsc = false;
        StudioBlockUIManager.blockUI("#manageItem", 400, 400, 0);

        if (!ASC.CRM.ListItemView.IsDropdownToggleRegistered) {
            ASC.CRM.ListItemView.IsDropdownToggleRegistered = true;
            if (ASC.CRM.ListItemView.CurrentType === 2 || ASC.CRM.ListItemView.CurrentType === 3) {
                jq.dropdownToggle({
                    dropdownID: "popup_iconsPanel_" + ASC.CRM.ListItemView.CurrentType,
                    switcherSelector: "#manageItem .change_icon",
                    noActiveSwitcherSelector: "#listView label.currentIcon",
                    addTop: 1,
                    addLeft: jq("#manageItem .change_icon").width() - 10,
                    position: "fixed"
                });
            }
            if (ASC.CRM.ListItemView.CurrentType === 1) {
                jq.dropdownToggle({
                    dropdownID: "popup_colorsPanel",
                    switcherSelector: "#manageItem .change_color",
                    noActiveSwitcherSelector: "#listView .currentColor",
                    addTop: 2,
                    addLeft: jq("#manageItem .change_color").width() - 10,
                    position: "fixed"
                });
            }
        }
    };

    var _changeColor = function(Obj, listItemId, color) {
        var data = {
            id: listItemId,
            color: color
        };
        Teamlab.updateCrmContactStatusColor({ Obj: Obj }, listItemId, data,
            function(params, contactStatus) {
                var index = _findIndexOfItemByID(contactStatus.id);
                if (index === -1) { return; }
                ASC.CRM.ListItemView.itemList[index] = contactStatus;

                jq(params.Obj).css("background-color", contactStatus.color);
                jq("#colorsPanel").hide();
            }
        );
    };

    var _changeIcon = function(Obj, listItemId, $imgObj) {
        var imgName = $imgObj.attr("img_name"),
            cssClass = imgName.split('.')[0],
            title = $imgObj.attr("title"),
            alt = $imgObj.attr("alt"),

            data = {
                id: listItemId,
                imageName: imgName
            };

        Teamlab.updateCrmListItemIcon({ Obj: jq(Obj), cssClass: cssClass, alt: alt, title: title }, ASC.CRM.ListItemView.CurrentType, listItemId, data, {
            before: function(params) {
                params.Obj.hide();
                params.Obj.parent().find("div.ajax_change_icon").show();
            },
            success: function(params, listItem) {
                var index = _findIndexOfItemByID(listItem.id);
                if (index === -1) { return; }
                ASC.CRM.ListItemView.itemList[index] = listItem;

                params.Obj.attr("class", ["currentIcon ", ASC.CRM.ListItemView.CurrentType == 2 ? "task_category" : "event_category", " ", params.cssClass].join(''));
                params.Obj.attr("title", params.title);
                params.Obj.attr("alt", params.alt);
                params.Obj.parent().find("div.ajax_change_icon").hide();

                jq(params.Obj).show();
                jq(".iconsPanelSettings").hide();
            }
        });
    };

    var _getIconByCssClass = function (cssClass) {
        var $icon = jq('#iconsPanel_' + ASC.CRM.ListItemView.CurrentType + ' label.' + cssClass);
        if ($icon.length != 1) {
            return null;
        }
        return $icon;
    };

    var _itemFactory = function(item) {
        if (typeof (item.relativeItemsCount) === "undefined") {
            item.relativeItemsCount = 0;
        }
        item.relativeItemsString = ASC.CRM.SettingsPage.getLinkContactStringByIndex(item.relativeItemsCount, jq.getURLParam("type"), null);
        if (ASC.CRM.ListItemView.CurrentType === 2 || ASC.CRM.ListItemView.CurrentType === 3) {
            if (item.hasOwnProperty("imagePath") && item.imagePath != "") {
                item.cssClass = item.imagePath.split('/')[item.imagePath.split('/').length - 1].split('.')[0];
                var $icon = _getIconByCssClass(item.cssClass);

                if ($icon != null) {
                    item.imageTitle = $icon.attr('title');
                    item.imageAlt = $icon.attr('alt');
                    item.imageName = $icon.attr('img_name');
                } else {
                    item.imageTitle = "";
                    item.imageAlt = "";
                    item.imageName = "";
                }
                
            } else {
                item.imageTitle = "";
                item.imageAlt = "";
                item.imageName = "";
                item.cssClass= "";
            }
        }

        var hash = [],
            filtervaluehash = {},
            anchor = "";

        if (ASC.CRM.ListItemView.CurrentType == 1) { //Contact types
            //hash.push('{"id":"sorter","type":"sorter","params":"eyJpZCI6ImRpc3BsYXluYW1lIiwiZGVmIjp0cnVlLCJkc2MiOmZhbHNlLCJzb3J0T3JkZXIiOiJhc2NlbmRpbmcifQ=="}');

            filtervaluehash = {
                id: "contactStage",
                type: "combobox",
                params: jq.base64.encode(jq.toJSON({
                    value: item.id,
                    title: item.title
                }))
            };

            hash.push(jq.toJSON(filtervaluehash));
            anchor = jq.base64.encode(hash.join(';'));
            item.relativeItemsUrl = "default.aspx#" + anchor;
        }
        if (ASC.CRM.ListItemView.CurrentType == 2) { //Task category
            //hash.push('{"id":"sorter","type":"sorter","params":"eyJpZCI6ImNhdGVnb3J5IiwiZGVmIjpmYWxzZSwiZHNjIjpmYWxzZSwic29ydE9yZGVyIjoiYXNjZW5kaW5nIn0="}');

            filtervaluehash = {
                id: "categoryID",
                type: "combobox",
                params: jq.base64.encode(jq.toJSON({
                    value: item.id,
                    title: item.title
                }))
            };

            hash.push(jq.toJSON(filtervaluehash));
            anchor = jq.base64.encode(hash.join(';'));
            item.relativeItemsUrl = "tasks.aspx#" + anchor;
        }

        if (ASC.CRM.ListItemView.CurrentType == 4) { //Contact type
            //hash.push('{"id":"sorter","type":"sorter","params":"eyJpZCI6ImNhdGVnb3J5IiwiZGVmIjpmYWxzZSwiZHNjIjpmYWxzZSwic29ydE9yZGVyIjoiYXNjZW5kaW5nIn0="}');

            filtervaluehash = {
                id: "contactType",
                type: "combobox",
                params: jq.base64.encode(jq.toJSON({
                    value: item.id,
                    title: item.title
                }))
            };

            hash.push(jq.toJSON(filtervaluehash));
            anchor = jq.base64.encode(hash.join(';'));
            item.relativeItemsUrl = "default.aspx#" + anchor;
        }
    };

    var _validateDataItem = function() {
        if (jq("#manageItem input:first").val().trim() == "") {
            ShowRequiredError(jq("#manageItem input:first"), true);
            return false;
        } else {
            RemoveRequiredErrorClass(jq("#manageItem input:first"));
            return true;
        }
    };

    var _readItemData = function(sortOrder) {
        var item = {
            title: jq("#manageItem input:first").val().trim(),
            sortOrder: sortOrder
        };
        if (ASC.CRM.ListItemView.CurrentType !== 4) {
            item.description = jq("#manageItem textarea").val().trim();
        }

        if (ASC.CRM.ListItemView.CurrentType === 1) {
            item.color = ASC.CRM.Common.getHexRGBColor(jq("#manageItem .selectedColor").css("background-color"));
        }
        if (ASC.CRM.ListItemView.CurrentType === 2 || ASC.CRM.ListItemView.CurrentType === 3) {
            item.imageName = jq("#manageItem label.selectedIcon").attr('img_name');
        }

        return item;
    };

    var _createItem = function() {
        if (!_validateDataItem()) { return; }

        var $listItems = jq("#listView li"),
            item = _readItemData($listItems.length),
            index = 0;

        for (var i = 0, len = $listItems.length; i < len; i++) {
            if (jq($listItems[i]).find("td.item_title").text().trim() == item.title) {
                index = i + 1;
                break;
            }
        }
        if (index == 0) {
            Teamlab.addCrmListItem({}, ASC.CRM.ListItemView.CurrentType, item, {
                before: function(params) {
                    jq("#manageItem .crm-actionButtonsBlock").hide();
                    jq("#manageItem .crm-actionProcessInfoBlock").show();
                },
                after: function(params) {
                    jq("#manageItem .crm-actionProcessInfoBlock").hide();
                    jq("#manageItem .crm-actionButtonsBlock").show();
                },
                success: ASC.CRM.ListItemView.CallbackMethods.add_item
            });
        } else {
            ASC.CRM.Common.animateElement({
                element: jq($listItems[index - 1]),
                afterScrollFunc: jq.unblockUI()
            });
        }
    };

    var _editItem = function(liObj, id, indexOfItem) {
        if (!_validateDataItem()) return;

        var $listItems = jq("#listView li"),
            item = _readItemData(indexOfItem),
            index = 0;

        item.id = id;
        for (var i = 0, len = $listItems.length; i < len; i++) {
            if (jq($listItems[i]).find("td.item_title").text().trim() == item.title && jq($listItems[i]).not(jq(liObj)).length != 0) {
                index = i + 1;
                break;
            }
        }
        if (index == 0) {
            Teamlab.updateCrmListItem({ liObj: liObj, indexOfItem: indexOfItem }, ASC.CRM.ListItemView.CurrentType, item.id, item, {
                before: function(params) {
                    jq("#manageItem .crm-actionButtonsBlock").hide();
                    jq("#manageItem .crm-actionProcessInfoBlock").show();
                },
                after: function(params) {
                    jq("#manageItem .crm-actionProcessInfoBlock").hide();
                    jq("#manageItem .crm-actionButtonsBlock").show();
                },
                success: ASC.CRM.ListItemView.CallbackMethods.edit_item
            });
        } else {
            ASC.CRM.Common.animateElement({
                element: jq($listItems[index - 1]),
                afterScrollFunc: jq.unblockUI()
            });
        }
    };

    return {
        CallbackMethods: {
            delete_item: function(params, item) {
                params.liObj.remove();

                var index = _findIndexOfItemByID(item.id);
                if (index != -1) {
                    ASC.CRM.ListItemView.itemList.splice(index, 1);
                }

                if (jq("#listView li").length == 1) {
                    jq(jq("#listItemActionMenu>.dropdown-content>li")[1]).hide();
                }
            },

            add_item: function(params, item) {
                _itemFactory(item);
                ASC.CRM.ListItemView.itemList.push(item);

                var $itemHTML = jq.tmpl("listItemsTmpl", item);
                if (jq("#listView li").length == 1) {
                    jq(jq("#listItemActionMenu>.dropdown-content>li")[1]).show();
                }
                $itemHTML.appendTo("#listView");

                ASC.CRM.Common.animateElement({
                    element: $itemHTML,
                    afterScrollFunc: jq.unblockUI()
                });
            },

            edit_item: function(params, item) {
                _itemFactory(item);
                ASC.CRM.ListItemView.itemList[params.indexOfItem] = item;

                var $itemHTML = jq.tmpl("listItemsTmpl", item);
                jq(params.liObj).hide();
                $itemHTML.insertAfter(params.liObj);
                jq(params.liObj).remove();

                ASC.CRM.Common.animateElement({
                    element: $itemHTML,
                    afterScrollFunc: jq.unblockUI()
                });
            }
        },

        init: function(currentType) {
            ASC.CRM.ListItemView.CurrentType = currentType;

            if (typeof (window.itemList) != "undefined") {
                window.itemList = jq.parseJSON(jQuery.base64.decode(window.itemList)).response;
            } else {
                window.itemList = [];
            }
            //itemList = Teamlab.create('crm-listitem', null, itemList);

            for (var i = 0, len = window.itemList.length; i < len; i++) {
                _itemFactory(window.itemList[i]);
            }
            ASC.CRM.ListItemView.itemList = window.itemList;

            jq.tmpl("listItemsTmpl", ASC.CRM.ListItemView.itemList).appendTo("#listView");

            if (ASC.CRM.ListItemView.itemList.length == 1) {
                jq(jq("#listItemActionMenu>.dropdown-content>li")[1]).hide();
            }

            _initSortableElements();
            _initElementsActionMenu();
            _initOtherActionMenu();

            if (jq("#cbx_ChangeContactStatusWithoutAsking").length == 1) {
                jq("#cbx_ChangeContactStatusWithoutAsking").bind("change", function () {
                    var changeContactStatusWithoutAsking = jq(this).is(":checked") ? true : null;
                    Teamlab.updateCRMContactStatusSettings({}, changeContactStatusWithoutAsking,
                        function () {
                        });
                });
            }

            jq("#createNewItem").bind("click", function() { _showAddItemPanel(); });

            ASC.CRM.ListItemView.IsDropdownToggleRegistered = false;
            if (ASC.CRM.ListItemView.CurrentType === 2 || ASC.CRM.ListItemView.CurrentType === 3) {
                jq.dropdownToggle({
                    dropdownID: "iconsPanel_" + ASC.CRM.ListItemView.CurrentType,
                    switcherSelector: "#listView label.currentIcon",
                    noActiveSwitcherSelector: "#manageItem .change_icon",
                    addTop: 1,
                    addLeft: -7,
                    showFunction: function(switcherObj, dropdownItem) {
                        if (dropdownItem.is(":hidden")) {
                            switcherObj.addClass("active");
                        } else {
                            switcherObj.removeClass("active");
                        }
                    },
                    hideFunction: function() {
                        var $activeImg = jq("#listView label.currentIcon.active");
                        if ($activeImg.length != 0) {
                            $activeImg.removeClass("active");
                        }
                    }
                });
            }
            if (ASC.CRM.ListItemView.CurrentType === 1) {
                jq.dropdownToggle({
                    dropdownID: "colorsPanel",
                    switcherSelector: "#listView .currentColor",
                    noActiveSwitcherSelector: "#manageItem .change_color",
                    addTop: 2,
                    addLeft: -13
                });
            }

            if (ASC.CRM.ListItemView.CurrentType === 2 || ASC.CRM.ListItemView.CurrentType === 3) {
                ASC.CRM.Common.registerChangeHoverStateByParent("label.currentIcon", "#listView .with-entity-menu");
                ASC.CRM.Common.registerChangeHoverStateByParent(".iconsPanelSettings>label", "");
            }
        },

        showEditItemPanel: function() {
            jq("#listItemActionMenu").hide();
            jq("#listView .entity-menu.active").removeClass("active");

            var index = 0,
                item = null,
                liObj = null,
                currIcon = null,
                listitemid = jq("#listItemActionMenu").attr("listitemid");
            if (typeof (listitemid) === "undefined" || listitemid == "") { return; }

            index = _findIndexOfItemByID(listitemid);
            if (index === -1) { return; }
            item = ASC.CRM.ListItemView.itemList[index];

            liObj = jq("#list_item_id_" + listitemid);
            if (liObj.length != 1) { return; }

            jq('#manageItem div.containerHeaderBlock td:first').text(ASC.CRM.ListItemView.EditItemHeaderText.replace('{0}', item.title));
            jq('#manageItem .crm-actionButtonsBlock a.button.blue.middle').text(ASC.CRM.Resources.CRMJSResource.SaveChanges);

            jq('#manageItem .crm-actionProcessInfoBlock > span').text(ASC.CRM.ListItemView.EditItemProcessText);
            jq('#manageItem .crm-actionProcessInfoBlock > img').attr("alt", ASC.CRM.ListItemView.EditItemProcessText);

            jq('#manageItem input:first').val(item.title);
            jq('#manageItem textarea').val(item.description);
            jq('#manageItem div.add_params input').val(item.additionalParams);

            currIcon = jq(liObj).find("label.currentIcon");
            if (currIcon.length == 1) {
                var $selectedIcinObj = jq("#manageItem label.selectedIcon"),
                    cssClass = jq(currIcon).attr("img_name").split('.')[0];

                $selectedIcinObj.attr('img_name', jq(currIcon).attr("img_name"));
                $selectedIcinObj.attr('title', jq(currIcon).attr('title'));
                $selectedIcinObj.attr('alt', jq(currIcon).attr('alt'));

                $selectedIcinObj.attr("class", ["selectedIcon ", ASC.CRM.ListItemView.CurrentType == 2 ? "task_category" : "event_category", " ", cssClass].join(''));
            } else {
                var currentColor = jq(liObj).find("div.currentColor").css("background-color");
                jq("#manageItem .selectedColor").css("background-color", currentColor);
            }

            jq('#manageItem .crm-actionButtonsBlock a.button.blue.middle').unbind('click').click(function() {
                _editItem(liObj, item.id, index);
            });
            RemoveRequiredErrorClass(jq("#manageItem input:first"));
            PopupKeyUpActionProvider.CloseDialogAction = "javascript:jq('.iconsPanelSettings').hide();jq('#colorsPanel').hide();jq('#popup_colorsPanel').hide();";
            PopupKeyUpActionProvider.EnableEsc = false;
            StudioBlockUIManager.blockUI("#manageItem", 400, 400, 0);

            if (!ASC.CRM.ListItemView.IsDropdownToggleRegistered) {
                ASC.CRM.ListItemView.IsDropdownToggleRegistered = true;
                if (ASC.CRM.ListItemView.CurrentType === 2 || ASC.CRM.ListItemView.CurrentType === 3) {
                    jq.dropdownToggle({
                        dropdownID: "popup_iconsPanel_" + ASC.CRM.ListItemView.CurrentType,
                        switcherSelector: "#manageItem .change_icon",
                        noActiveSwitcherSelector: "#listView label.currentIcon",
                        addTop: 1,
                        addLeft: jq("#manageItem .change_icon").width() - 10,
                        position: "fixed"
                    });
                }
                if (ASC.CRM.ListItemView.CurrentType === 1) {
                    jq.dropdownToggle({
                        dropdownID: "popup_colorsPanel",
                        switcherSelector: "#manageItem .change_color",
                        noActiveSwitcherSelector: "#listView .currentColor",
                        addTop: 2,
                        addLeft: jq("#manageItem .change_color").width() - 10,
                        position: "fixed"
                    });
                }
            }
        },

        deleteItem: function() {
            jq("#listItemActionMenu").hide();
            jq("#listView .entity-menu.active").removeClass("active");

            var listitemid = jq("#listItemActionMenu").attr('listitemid');
            if (typeof (listitemid) === "undefined" || listitemid == "") { return; }

            var liObj = jq("#list_item_id_" + listitemid);
            if (liObj.length != 1) { return; }

            if (jq("#listView li").length == 1) {
                if (ASC.CRM.ListItemView.CurrentType == 1) {
                    alert(jq.format(ASC.CRM.Resources.CRMJSResource.ErrorTheLastContactStage,
                        jq(liObj).find(".item_title").text()) + "\n" + ASC.CRM.Resources.CRMJSResource.PleaseRefreshThePage);
                    jq("#listView").find(".entity-menu").hide();
                    return;
                }
                if (ASC.CRM.ListItemView.CurrentType == 2) {
                    alert(jq.format(ASC.CRM.Resources.CRMJSResource.ErrorTheLastTaskCategory,
                        jq(liObj).find(".item_title").text()) + "\n" + ASC.CRM.Resources.CRMJSResource.PleaseRefreshThePage);
                    jq("#listView").find(".entity-menu").hide();
                    return;
                }
                return;
            }

            Teamlab.removeCrmListItem({ liObj: liObj }, ASC.CRM.ListItemView.CurrentType, listitemid, {
                before: function(params) {
                    params.liObj.find(".entity-menu").hide();
                    params.liObj.find("div.ajax_loader").show();
                },
                success: ASC.CRM.ListItemView.CallbackMethods.delete_item
            });
        },

        showColorsPanel: function(switcherUI) {
            jq("#colorsPanel > span").unbind("click").click(function() {
                _changeColor(switcherUI, jq(switcherUI).parents("li").get(0).id.replace("list_item_id_", "") * 1, ASC.CRM.Common.getHexRGBColor(jq(this).css("background-color")));
            });
        },

        showColorsPanelToSelect: function() {
            jq("#popup_colorsPanel > span").unbind("click").click(function() {
                jq("#manageItem .selectedColor").css("background", ASC.CRM.Common.getHexRGBColor(jq(this).css("background-color")));
                jq("#popup_colorsPanel").hide();
            });
        },

        showIconsPanel: function(switcherUI) {
            var $iconsPanel = jq("#iconsPanel_" + ASC.CRM.ListItemView.CurrentType);
            if ($iconsPanel.length != 1) return;
            $iconsPanel.children("label").unbind("click").click(function() {
                _changeIcon(switcherUI, jq(switcherUI).parents("li").get(0).id.replace("list_item_id_", "") * 1, jq(this));
            });
        },

        showIconsPanelToSelect: function() {
            var $popup_iconsPanel = jq("#popup_iconsPanel_" + ASC.CRM.ListItemView.CurrentType);
            if ($popup_iconsPanel.length != 1) return;
            $popup_iconsPanel.children("label").unbind("click").click(function() {
                var $selectedIcinObj = jq("#manageItem label.selectedIcon"),
                    cssClass = jq(this).attr('img_name').split('.')[0];

                $selectedIcinObj.attr('img_name', jq(this).attr('img_name'));
                $selectedIcinObj.attr('title', jq(this).attr('title'));
                $selectedIcinObj.attr('alt', jq(this).attr('alt'));
                $selectedIcinObj.attr("class", ["selectedIcon ", ASC.CRM.ListItemView.CurrentType == 2 ? "task_category" : "event_category", " ", cssClass].join(''));

                $popup_iconsPanel.hide();
            });
        }
    };
})();


ASC.CRM.DealMilestoneView = (function() {

    var _initSortableDealMilestones = function() {
        jq("#dealMilestoneList").sortable({
            cursor: "move",
            handle: ".sort_drag_handle",
            items: "li",
            start: function(event, ui) {
                jq("#colorsPanel").hide();
                jq("#popup_colorsPanel").hide();
                jq("#dealMilestoneActionMenu").hide();
                jq("#dealMilestoneList .entity-menu.active").removeClass("active");
            },
            beforeStop: function(event, ui) {
                jq(ui.item[0]).attr("style", "");
            },
            update: function(event, ui) {
                var itemsIDs = [];

                jq("#dealMilestoneList li[id^='deal_milestone_id_']").each(function() {
                    itemsIDs.push(jq(this).attr("id").replace("deal_milestone_id_", "") * 1);
                });

                Teamlab.reorderCrmDealMilestones({}, itemsIDs, {});
            }
        });
    };

    var _initDealMilestonesActionMenu = function() {

        jq.dropdownToggle({
            dropdownID: "dealMilestoneActionMenu",
            switcherSelector: "#dealMilestoneList .entity-menu",
            addTop: -2,
            addLeft: 10,
            rightPos: true,
            showFunction: function(switcherObj, dropdownItem) {
                jq("#dealMilestoneList .entity-menu.active").removeClass("active");
                if (dropdownItem.is(":hidden")) {
                    switcherObj.addClass("active");
                    if (switcherObj.attr("dealmilestoneid") != dropdownItem.attr("dealmilestoneid")) {
                        dropdownItem.attr('dealmilestoneid', switcherObj.attr("dealmilestoneid"));
                    }
                }
            },
            hideFunction: function() {
                jq("#dealMilestoneList .entity-menu.active").removeClass("active");
            }
        });
    };

    var _initOtherActionMenu = function() {
        ASC.CRM.Common.removeExportButtons();
        jq("#menuCreateNewTask").bind("click", function() { ASC.CRM.TaskActionView.showTaskPanel(0, "", 0, null, {}); });
    };

    var _resetManageItemPanel = function() {
        jq("#manageDealMilestone .title").val("");
        jq("#manageDealMilestone .probability").val("0");
        jq("#manageDealMilestone textarea").val("");
        jq("#manageDealMilestone [name=deal_milestone_status]:first").prop("checked", true);

        var $divColors = jq("#colorsPanel > span"),
            ind = Math.floor(Math.random() * $divColors.length);
        jq("#manageDealMilestone .selectedColor").css("background-color", ASC.CRM.Common.getHexRGBColor(jq($divColors.get(ind)).css("background-color")));
    };

    var _showAddDealMilestonePanel = function() {
        jq('#manageDealMilestone div.containerHeaderBlock td:first').text(ASC.CRM.DealMilestoneView.AddDealMilestoneHeaderText);
        jq('#manageDealMilestone .crm-actionButtonsBlock a.button.blue.middle').text(ASC.CRM.DealMilestoneView.AddDealMilestoneButtonText);
        jq('#manageDealMilestone .crm-actionProcessInfoBlock > span').text(ASC.CRM.DealMilestoneView.AddDealMilestoneProcessText);
        jq('#manageDealMilestone .crm-actionProcessInfoBlock > img').attr("alt", ASC.CRM.DealMilestoneView.AddDealMilestoneProcessText);
        _resetManageItemPanel();

        jq('#manageDealMilestone .crm-actionButtonsBlock a.button.blue.middle').unbind('click').click(function() {
            _createDealMilestone();
        });
        RemoveRequiredErrorClass(jq("#manageDealMilestone .title"));

        PopupKeyUpActionProvider.EnableEsc = false;
        StudioBlockUIManager.blockUI("#manageDealMilestone", 400, 400, 0);

        if (!ASC.CRM.DealMilestoneView.IsDropdownToggleRegistered) {
            ASC.CRM.DealMilestoneView.IsDropdownToggleRegistered = true;
            jq.dropdownToggle({
                dropdownID: "popup_colorsPanel",
                switcherSelector: "#manageDealMilestone .change_color",
                noActiveSwitcherSelector: "#dealMilestoneList .currentColor",
                addTop: 2,
                addLeft: jq("#manageDealMilestone .change_color").width() - 10,
                position: "fixed"
            });
        }
    };

    var _findIndexOfDealMilestoneByID = function(id) {
        for (var i = 0, n = ASC.CRM.DealMilestoneView.dealMilestoneList.length; i < n; i++) {
            if (ASC.CRM.DealMilestoneView.dealMilestoneList[i].id == id) {
                return i;
            }
        }
        return -1;
    };

    var _changeColor = function(Obj, listItemId, color) {
        var data = {
            id: listItemId,
            color: color
        };
        Teamlab.updateCrmDealMilestoneColor({ Obj: Obj }, listItemId, data,
            function(params, dealMilestone) {
                jq(params.Obj).css("background", dealMilestone.color);
                jq("#colorsPanel").hide();
            }
        );
    };

    var _dealMilestoneItemFactory = function(dealMilestone) {
        if (typeof (dealMilestone.relativeItemsCount) === "undefined") {
            dealMilestone.relativeItemsCount = 0;
        }
        dealMilestone.relativeItemsString = ASC.CRM.SettingsPage.getLinkContactStringByIndex(dealMilestone.relativeItemsCount, jq.getURLParam("type"), null);
        var hash = [];
        hash.push('{"id":"sorter","type":"sorter","params":"eyJpZCI6InN0YWdlIiwiZGVmIjp0cnVlLCJkc2MiOmZhbHNlLCJzb3J0T3JkZXIiOiJhc2NlbmRpbmcifQ=="}');

        var filtervaluehash = {
            id: "opportunityStagesID",
            type: "combobox",
            params: jq.base64.encode(jq.toJSON({
                value: dealMilestone.id,
                title: dealMilestone.title
            }))
        };

        hash.push(jq.toJSON(filtervaluehash));
        var anchor = jq.base64.encode(hash.join(';'));
        dealMilestone.relativeItemsUrl = "deals.aspx#" + anchor;
    };

    var _readDealMilestoneData = function(sortOrder) {
        var dealMilestone =
        {
            title: jq("#manageDealMilestone .title").val().trim(),
            sortOrder: sortOrder
        };

        dealMilestone.description = jq("#manageDealMilestone dl textarea").val().trim();
        dealMilestone.color = ASC.CRM.Common.getHexRGBColor(jq("#manageDealMilestone .selectedColor").css("background-color"));

        var percent = jq("#manageDealMilestone .probability").val();
        if (percent * 1 > 100) { percent = "100"; }

        dealMilestone.successProbability = percent;
        dealMilestone.stageType = jq("#manageDealMilestone [name=deal_milestone_status]:checked").val() * 1;

        return dealMilestone;
    };

    var _editDealMilestone = function(liObj, id) {
        if (jq("#manageDealMilestone .title").val().trim() == "") {
            ShowRequiredError(jq("#manageDealMilestone .title"), true);
            return;
        } else {
            RemoveRequiredErrorClass(jq("#manageDealMilestone .title"));
        }

        var $listItems = jq("#dealMilestoneList li"),
            dealMilestone = _readDealMilestoneData($listItems.length),
            index = 0;

        dealMilestone.id = id;

        for (var i = 0, len = $listItems.length; i < len; i++) {
            if (jq($listItems[i]).find("td.deal_milestone_title").text().trim() == dealMilestone.title && jq($listItems[i]).not(jq(liObj)).length != 0) {
                index = i + 1;
                break;
            }
        }
        if (index == 0) {
            Teamlab.updateCrmDealMilestone({ liObj: liObj }, dealMilestone.id, dealMilestone, {
                before: function() {
                    jq("#manageDealMilestone .crm-actionButtonsBlock").hide();
                    jq("#manageDealMilestone .crm-actionProcessInfoBlock").show();
                },
                after: function() {
                    jq("#manageDealMilestone .crm-actionProcessInfoBlock").hide();
                    jq("#manageDealMilestone .crm-actionButtonsBlock").show();
                },
                success: ASC.CRM.DealMilestoneView.CallbackMethods.edit_dealMilestone
            });
        } else {
            ASC.CRM.Common.animateElement({
                element: jq($listItems[index - 1]),
                afterScrollFunc: jq.unblockUI()
            });
        }
    };

    var _createDealMilestone = function() {
        if (jq("#manageDealMilestone .title").val().trim() == "") {
            ShowRequiredError(jq("#manageDealMilestone .title"), true);
            return;
        } else {
            RemoveRequiredErrorClass(jq("#manageDealMilestone .title"));
        }

        var $listItems = jq("#dealMilestoneList li"),
            dealMilestone = _readDealMilestoneData($listItems.length),
            index = 0;

        for (var i = 0, len = $listItems.length; i < len; i++) {
            if (jq($listItems[i]).find("td.deal_milestone_title").text().trim() == dealMilestone.title) {
                index = i + 1;
                break;
            }
        }

        if (index == 0) {
            Teamlab.addCrmDealMilestone({}, dealMilestone, {
                before: function() {
                    jq("#manageDealMilestone .crm-actionButtonsBlock").hide();
                    jq("#manageDealMilestone .crm-actionProcessInfoBlock").show();
                },
                after: function() {
                    jq("#manageDealMilestone .crm-actionProcessInfoBlock").hide();
                    jq("#manageDealMilestone .crm-actionButtonsBlock").show();
                },
                success: ASC.CRM.DealMilestoneView.CallbackMethods.add_dealMilestone
            });
        } else {
            ASC.CRM.Common.animateElement({
                element: jq($listItems[index - 1]),
                afterScrollFunc: jq.unblockUI()
            });
        }
    };

    return {
        CallbackMethods: {
            add_dealMilestone: function(params, dealMilestone) {
                _dealMilestoneItemFactory(dealMilestone);
                ASC.CRM.DealMilestoneView.dealMilestoneList.push(dealMilestone);

                if (jq("#dealMilestoneList li").length == 1) {
                    jq(jq("#dealMilestoneActionMenu>.dropdown-content>li")[1]).show();
                }

                var $itemHTML = jq.tmpl("dealMilestoneTmpl", dealMilestone);
                $itemHTML.appendTo("#dealMilestoneList");

                ASC.CRM.Common.animateElement({
                    element: $itemHTML,
                    afterScrollFunc: jq.unblockUI()
                });
            },

            edit_dealMilestone: function(params, dealMilestone) {
                _dealMilestoneItemFactory(dealMilestone);

                var index = _findIndexOfDealMilestoneByID(dealMilestone.id);
                if (index != -1) {
                    ASC.CRM.DealMilestoneView.dealMilestoneList[index] = dealMilestone;
                }

                var $itemHTML = jq.tmpl("dealMilestoneTmpl", dealMilestone);

                params.liObj.hide();
                $itemHTML.insertAfter(params.liObj);
                params.liObj.remove();

                ASC.CRM.Common.animateElement({
                    element: $itemHTML,
                    afterScrollFunc: jq.unblockUI()
                });
            },

            delete_dealMilestone: function(params, dealMilestone) {
                params.liObj.remove();
                var index = _findIndexOfDealMilestoneByID(dealMilestone.id);
                if (index != -1) {
                    ASC.CRM.DealMilestoneView.dealMilestoneList.splice(index, 1);
                }
                if (jq("#dealMilestoneList li").length == 1) {
                    jq(jq("#dealMilestoneActionMenu>.dropdown-content>li")[1]).hide();
                }
            }
        },

        init: function() {
            if (typeof (window.dealMilestoneList) != "undefined") {
                window.dealMilestoneList = jq.parseJSON(jQuery.base64.decode(window.dealMilestoneList)).response;
            } else {
                window.dealMilestoneList = [];
            }
            ASC.CRM.DealMilestoneView.dealMilestoneList = Teamlab.create('crm-dealmilestone', null, window.dealMilestoneList);

            for (var i = 0, n = ASC.CRM.DealMilestoneView.dealMilestoneList.length; i < n; i++) {
                _dealMilestoneItemFactory(ASC.CRM.DealMilestoneView.dealMilestoneList[i]);
            }
            jq.tmpl("dealMilestoneTmpl", ASC.CRM.DealMilestoneView.dealMilestoneList).appendTo("#dealMilestoneList");

            if (ASC.CRM.DealMilestoneView.dealMilestoneList.length == 1) {
                jq(jq("#dealMilestoneActionMenu>.dropdown-content>li")[1]).hide();
            }

            _initSortableDealMilestones();
            _initDealMilestonesActionMenu();
            _initOtherActionMenu();
            jq("#createNewDealMilestone").bind("click", function() { _showAddDealMilestonePanel(); });

            ASC.CRM.DealMilestoneView.IsDropdownToggleRegistered = false;
            jq.dropdownToggle({
                dropdownID: "colorsPanel",
                switcherSelector: "#dealMilestoneList .currentColor",
                noActiveSwitcherSelector: "#manageDealMilestone .change_color",
                addTop: 2,
                addLeft: -13
            });

            jq.forceIntegerOnly("#manageDealMilestone .probability");
        },

        showEditDealMilestonePanel: function() {
            jq("#dealMilestoneActionMenu").hide();
            jq("#dealMilestoneList .entity-menu.active").removeClass("active");

            var dealmilestoneid = jq("#dealMilestoneActionMenu").attr("dealmilestoneid");
            if (typeof (dealmilestoneid) === "undefined" || dealmilestoneid == "") { return; }

            var liObj = jq("#deal_milestone_id_" + dealmilestoneid);
            if (liObj.length != 1) { return; }

            var index = _findIndexOfDealMilestoneByID(dealmilestoneid);
            if (index === -1) { return; }
            var dealMilestone = ASC.CRM.DealMilestoneView.dealMilestoneList[index];


            jq('#manageDealMilestone div.containerHeaderBlock td:first').text(ASC.CRM.DealMilestoneView.EditDealMilestoneHeaderText.replace('{0}', dealMilestone.title));
            jq('#manageDealMilestone .crm-actionButtonsBlock a.button.blue.middle').text(ASC.CRM.Resources.CRMJSResource.SaveChanges);
            jq('#manageDealMilestone .crm-actionProcessInfoBlock > span').text(ASC.CRM.DealMilestoneView.EditDealMilestoneProcessText);
            jq('#manageDealMilestone .crm-actionProcessInfoBlock > img').attr("alt", ASC.CRM.DealMilestoneView.EditDealMilestoneProcessText);


            jq('#manageDealMilestone .title').val(dealMilestone.title);
            jq('#manageDealMilestone textarea').val(dealMilestone.description);
            jq('#manageDealMilestone .probability').val(dealMilestone.successProbability);
            jq("#manageDealMilestone [name=deal_milestone_status][value=" + dealMilestone.stageType + "]").prop("checked", true);

            jq('#manageDealMilestone .crm-actionButtonsBlock a.button.blue.middle').unbind('click').click(function() {
                _editDealMilestone(liObj, dealMilestone.id);
            });
            jq("#manageDealMilestone .selectedColor").css("background-color", dealMilestone.color);

            RemoveRequiredErrorClass(jq("#manageDealMilestone .title"));
            PopupKeyUpActionProvider.EnableEsc = false;
            StudioBlockUIManager.blockUI("#manageDealMilestone", 400, 400, 0);

            if (!ASC.CRM.DealMilestoneView.IsDropdownToggleRegistered) {
                ASC.CRM.DealMilestoneView.IsDropdownToggleRegistered = true;
                jq.dropdownToggle({
                    dropdownID: "popup_colorsPanel",
                    switcherSelector: "#manageDealMilestone .change_color",
                    noActiveSwitcherSelector: "#dealMilestoneList .currentColor",
                    addTop: 2,
                    addLeft: jq("#manageDealMilestone .change_color").width() - 10,
                    position: "fixed"
                });
            }
        },

        deleteDealMilestone: function() {
            jq("#dealMilestoneActionMenu").hide();
            jq("#dealMilestoneList .entity-menu.active").removeClass("active");

            var dealmilestoneid = jq("#dealMilestoneActionMenu").attr("dealmilestoneid");
            if (typeof (dealmilestoneid) === "undefined" || dealmilestoneid == "") { return; }

            var liObj = jq("#deal_milestone_id_" + dealmilestoneid);
            if (liObj.length != 1) { return; }

            if (jq("#dealMilestoneList li").length == 1) {
                alert(ASC.CRM.Resources.CRMJSResource.ErrorTheLastDealMilestone);
                return;
            }

            Teamlab.removeCrmDealMilestone({ liObj: liObj }, dealmilestoneid, {
                before: function(params) {
                    params.liObj.find(".entity-menu").hide();
                    params.liObj.find("div.ajax_loader").show();
                },
                success: ASC.CRM.DealMilestoneView.CallbackMethods.delete_dealMilestone
            });
        },

        showColorsPanel: function(switcherUI) {
            jq("#colorsPanel > span").unbind("click").click(function() {
                _changeColor(switcherUI, jq(switcherUI).parents('li').get(0).id.replace("deal_milestone_id_", "") * 1, ASC.CRM.Common.getHexRGBColor(jq(this).css("background-color")));
            });
        },

        showColorsPanelToSelect: function() {
            jq("#popup_colorsPanel > span").unbind("click").click(function() {
                jq("#manageDealMilestone .selectedColor").css("background", ASC.CRM.Common.getHexRGBColor(jq(this).css("background-color")));
                jq("#popup_colorsPanel").hide();
            });
        }
    };
})();


ASC.CRM.TagSettingsView = (function() {

    var _initOtherActionMenu = function() {
        ASC.CRM.Common.removeExportButtons();
        if (jq("#otherActions").length == 1) {
            jq.tmpl("deleteUnusedTagsButtonTmpl").appendTo("#otherActions ul.dropdown-content");
            jq("#deleteUnusedTagsButton").bind("click", function() { _deleteUnusedTags(); });
        }

        jq("#menuCreateNewTask").bind("click", function() { ASC.CRM.TaskActionView.showTaskPanel(0, "", 0, null, {}); });
    };

    var _showAddTagPanel = function() {
        jq("#tagTitle").val("");
        RemoveRequiredErrorClass(jq("#tagTitle"));
        PopupKeyUpActionProvider.EnableEsc = false;
        StudioBlockUIManager.blockUI("#manageTag", 400, 400, 0);
    };

    var _deleteUnusedTags = function() {
        var view = jq.getURLParam("view"),
            entityType = view != null && view != ""
                        ? view
                        : "contact";

        jq("#otherActions").hide();
        jq("#deleteUnusedTagsButton").parent().addClass("display-none");

        Teamlab.removeCrmUnusedTag({}, entityType, {
            success: ASC.CRM.TagSettingsView.CallbackMethods.delete_unused_tags
        });
    };

    var _tagItemFactory = function(tag, count) {
        var view = jq.getURLParam("view");
        tag.relativeItemsCount = count;
        tag.relativeItemsString = ASC.CRM.SettingsPage.getLinkContactStringByIndex(count, jq.getURLParam("type"), view);

        var hash = [],
            filtervaluehash = {},
            anchor = "",
            baseUrl = "default.aspx";
        //hash.push('{"id":"sorter","type":"sorter","params":"eyJpZCI6InRpdGxlIiwiZGVmIjp0cnVlLCJkc2MiOmZhbHNlLCJzb3J0T3JkZXIiOiJhc2NlbmRpbmcifQ=="}');

        filtervaluehash = {
            id: "tags",
            type: "combobox",
            params: jq.base64.encode(jq.toJSON({
                value: [ASC.CRM.Common.convertText(tag.value,false)],
                title: ASC.CRM.Common.convertText(tag.value,false)
            }))
        };

        hash.push(jq.toJSON(filtervaluehash));
        anchor = jq.base64.encode(hash.join(';'));

        if (view == "") {
            baseUrl = "default.aspx";
        }
        if (view == "opportunity") {
            baseUrl = "deals.aspx";
        }
        if (view == "case") {
            baseUrl = "cases.aspx";
        }
        tag.relativeItemsUrl = baseUrl + "#" + anchor;
    };

    var _findIndexOfTagByName = function(name) {
        for (var i = 0, n = window.tagList.length; i < n; i++) {
            if (window.tagList[i].value == name) {
                return i;
            }
        }
        return -1;
    };

    var _doUnusedTagsExist = function() {
        for (var i = 0, n = window.tagList.length; i < n; i++) {
            if (window.tagList[i].relativeItemsCount == 0) {
                return true;
            }
        }
        return false;
    };

    return {
        CallbackMethods: {
            add_tag: function(params, tag) {
                tag = { value: tag };
                _tagItemFactory(tag, 0);

                window.tagList.push(tag);

                if (jq("#deleteUnusedTagsButton").length == 1) {
                    jq("#deleteUnusedTagsButton").parent().removeClass("display-none");
                }

                var $itemHTML = jq.tmpl("tagRowTemplate", tag);
                if (params.tagsCount == 0) {
                    jq("#emptyTagContent").hide();
                    jq("#tagList").show();
                    jq("#createNewTagSettings.display-none").removeClass("display-none");
                    jq(".cbx_AddTagWithoutAskingContainer.display-none").removeClass("display-none");
                }
                $itemHTML.appendTo("#tagList");

                ASC.CRM.Common.animateElement({
                    element: $itemHTML,
                    afterScrollFunc: jq.unblockUI()
                });
            },

            delete_tag: function(params, tag) {
                params.liObj.remove();

                var index = _findIndexOfTagByName(tag);
                if (index != -1) {
                    window.tagList.splice(index, 1);
                }
                if (jq("#deleteUnusedTagsButton").length == 1 && !_doUnusedTagsExist()) {
                    jq("#deleteUnusedTagsButton").parent().addClass("display-none");
                }

                if (jq("#tagList li").length == 0) {
                    jq("#tagList").hide();
                    jq("#createNewTagSettings:not(.display-none)").addClass("display-none");
                    jq(".cbx_AddTagWithoutAskingContainer:not(.display-none)").addClass("display-none");
                    if (jq("#deleteUnusedTagsButton").length == 1) {
                        jq("#deleteUnusedTagsButton").parent().addClass("display-none");
                    }
                    jq("#emptyTagContent").show();
                }
            },

            delete_unused_tags: function(params, tags) {
                var $listTags = jq("#tagList li");

                for (var j = 0, n = tags.length; j < n; j++) {
                    for (var i = 0, m = $listTags.length; i < m; i++) {
                        if (jq($listTags[i]).find(".title").text().trim() == tags[j]) {
                            jq($listTags[i]).remove();
                        }
                    }
                    var index = _findIndexOfTagByName(tags[j]);
                    if (index != -1) {
                        window.tagList.splice(index, 1);
                    }
                }

                if (jq("#deleteUnusedTagsButton").length == 1 && !_doUnusedTagsExist()) {
                    jq("#deleteUnusedTagsButton").parent().addClass("display-none");
                }

                if (jq("#tagList li").length == 0) {
                    jq("#tagList").hide();
                    jq("#createNewTagSettings:not(.display-none)").addClass("display-none");
                    jq(".cbx_AddTagWithoutAskingContainer:not(.display-none)").addClass("display-none");
                    if (jq("#deleteUnusedTagsButton").length == 1) {
                        jq("#deleteUnusedTagsButton").parent().addClass("display-none");
                    }
                    jq("#emptyTagContent").show();
                }
            }
        },

        emptyErrorMessage: "",
        alreadyExistsErrorMessage: "",
        init: function(emptyErrorText, alreadyExistsErrorText) {
            ASC.CRM.TagSettingsView.emptyErrorMessage = emptyErrorText;
            ASC.CRM.TagSettingsView.alreadyExistsErrorMessage = alreadyExistsErrorText;
            //jq("#tagList").disableSelection();
            _initOtherActionMenu();
            jq("#createNewTagSettings").bind("click", function() { _showAddTagPanel(); });
            jq("#emptyTagContent .emptyScrBttnPnl > a").bind("click", function() { _showAddTagPanel(); });

            if (typeof (window.tagList) == "undefined" ||
                typeof (window.relativeItemsCountArray) == "undefined" ||
                window.tagList.length != window.relativeItemsCountArray.length)
            { return; }

            for (var i = 0, len = window.tagList.length; i < len; i++) {
                _tagItemFactory(window.tagList[i], window.relativeItemsCountArray[i]);
            }

            if (window.tagList.length != 0) {
                jq.tmpl("tagRowTemplate", window.tagList).appendTo("#tagList");
                jq("#tagList").show();
                jq("#createNewTagSettings.display-none").removeClass("display-none");
                jq(".cbx_AddTagWithoutAskingContainer.display-none").removeClass("display-none");
            } else {
                jq("#createNewTagSettings:not(.display-none)").addClass("display-none");
                jq(".cbx_AddTagWithoutAskingContainer:not(.display-none)").addClass("display-none");
                jq("#emptyTagContent").show();
            }
            if (jq("#deleteUnusedTagsButton").length == 1 && !_doUnusedTagsExist()) {
                jq("#deleteUnusedTagsButton").parent().addClass("display-none");
            }


            if (jq("#cbx_AddTagWithoutAsking").length == 1) {
                jq("#cbx_AddTagWithoutAsking").bind("change", function () {
                    var addTagWithoutAsking = jq(this).is(":checked") ? true : null;
                    Teamlab.updateCRMContactTagSettings({}, addTagWithoutAsking,
                        function () {
                        });
                });
            }
        },

        initEmptyScreen: function (imgSrc, headerText, describeText, buttonText) {
            jq.tmpl("emptyScrTmpl",
                {
                    ID: "emptyTagContent",
                    ImgSrc: imgSrc,
                    Header: headerText,
                    Describe: describeText,
                    ButtonHTML: ['<a id="addTag" class="link dotline blue plus">',buttonText,'</a>'].join('')
                }).insertAfter("#tagList");
        },

        createTag: function() {
            var tagTitle = jq.trim(jq("#tagTitle").val());
            RemoveRequiredErrorClass(jq("#tagTitle"));

            if (tagTitle == "") {
                AddRequiredErrorText(jq("#tagTitle"), ASC.CRM.TagSettingsView.emptyErrorMessage);
                ShowRequiredError(jq("#tagTitle"), true);
                return;
            }

            var $listTags = jq("#tagList li"),
                tagsCount = $listTags.length;

            for (var i = 0; i < tagsCount; i++) {
                if (jq($listTags[i]).find(".title").text().trim() == tagTitle) {
                    AddRequiredErrorText(jq("#tagTitle"), ASC.CRM.TagSettingsView.alreadyExistsErrorMessage);
                    ShowRequiredError(jq("#tagTitle"), true);
                    return;
                }
            }

            var view = jq.getURLParam("view"),
                entityType = view != null && view != ""
                            ? view
                            : "contact";

            Teamlab.addCrmEntityTag({ view: view, tagsCount: tagsCount }, entityType, tagTitle, {
                before: function(params) {
                    jq("#manageTag .crm-actionButtonsBlock").hide();
                    jq("#manageTag .crm-actionProcessInfoBlock").show();
                },
                success: ASC.CRM.TagSettingsView.CallbackMethods.add_tag,
                after: function() {
                    jq("#manageTag .crm-actionProcessInfoBlock").hide();
                    jq("#manageTag .crm-actionButtonsBlock").show();
                }
            });

        },

        deleteTag: function(deleteLink) {
            var liObj = jq(jq(deleteLink).parents('li').get(0)),

                tagTitle = liObj.find(".title").text().trim(),
                view = jq.getURLParam("view"),
                entityType = view != null && view != ""
                            ? view
                            : "contact";

            Teamlab.removeCrmEntityTag({ liObj: liObj }, entityType, Encoder.htmlDecode(tagTitle), {
                before: function(params) {
                    params.liObj.find("label.crm-deleteBtn").hide();
                    params.liObj.find("div.ajax_loader").show();
                },
                success: ASC.CRM.TagSettingsView.CallbackMethods.delete_tag
            });
        }
    };
})();


ASC.CRM.SettingsPage.WebToLeadFormView = (function() {
    var _initOtherActionMenu = function() {
        ASC.CRM.Common.removeExportButtons();

        jq("#menuCreateNewTask").bind("click", function () { ASC.CRM.TaskActionView.showTaskPanel(0, "", 0, null, {}); });
    };

    function _renderTable(serverData, container) {
        var residue = serverData.length % 3 > 0 ? 3 - serverData.length % 3 : 0,
            counter = 0,
            row,
            cell,
            input,
            label;
        var indexRow = 0,
            isCompany = jq("input[name=radio]:checked").val() == "company";

        jq(serverData).each(function() {
            if (!isCompany && (this.name == "firstName" || this.name == "lastName")) {
                this.disabled = true;
            } else if (isCompany && this.name == "companyName") {
                this.disabled = true;
            } else {
                this.disabled = false;
            }
        });

        for (var i = 0; i < serverData.length + residue; i++) {
            if (counter == 0) {
                row = jq("<tr></tr>");
            }
            cell = jq("<td></td>").attr("width", "33%");

            if (serverData[i] != null) {
                input = jq("<input>")
                                .attr("type", "checkbox")
                                .attr("id", serverData[i].name)
                                .attr("name", serverData[i].name);

                if (serverData[i].disabled) {
                    jq(input).prop("checked", true).prop("disabled", true);
                }

                label = jq("<label>").attr("for", serverData[i].name).text(serverData[i].title);
                input.data("fieldInfo", serverData[i]);
                cell.append(input).append(label);
            }

            row.append(cell);
            counter++;

            if (counter == 3 || i == serverData.length + residue - 1) {
                container.append(row);
                counter = 0;
                indexRow++;
            }
        }
    };

    function _validateInputData() {

        var regexp = /(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/;

        var returnURL = jq("#returnURL").val().trim(),
            webFormKey = jq("#properties_webFormKey input").val().trim(),
            isValid = true;

        if (returnURL == "" || !regexp.test(returnURL)) {
            ShowRequiredError(jq("#returnURL"));
            isValid = false;
        } else {
            RemoveRequiredErrorClass(jq("#returnURL"));
        }

        if (webFormKey == "") {
            ShowRequiredError(jq("#properties_webFormKey input"));
            isValid = false;
        } else {
            RemoveRequiredErrorClass(jq("#properties_webFormKey input"));
        }

        var firstName = jq("#tblFieldList input[name='firstName']:checked"),
            lastName = jq("#tblFieldList input[name='lastName']:checked"),
            companyName = jq("#tblFieldList input[name='companyName']:checked");

        if ((firstName.length == 0 || lastName.length == 0) && companyName.length == 0) {
            alert(ASC.CRM.Resources.CRMJSResource.ErrorNotMappingBasicColumn);
            isValid = false;
        }

        return isValid;
    };

    return {
        init: function (webtoleadfromhandlerPath) {
            if (webtoleadfromhandlerPath.indexOf("http:") == 0) {
                webtoleadfromhandlerPath = webtoleadfromhandlerPath.replace("http:", "https:");
            }

            ASC.CRM.SettingsPage.WebToLeadFormView.webtoleadfromhandlerPath = webtoleadfromhandlerPath;

            var tags = [];
            if (typeof(window.tagList) != "undefined") {
                for (var i = 0, n = window.tagList.length; i < n; i++)
                {
                    tags.push(Encoder.htmlDecode(window.tagList[i].title));
                }
            }

            jq.tmpl("tagViewTmpl",
                    {
                        tags          : [],
                        availableTags : tags
                    })
                    .appendTo("#wtlfTags");
            ASC.CRM.TagView.init("contact", true);


            ASC.CRM.UserSelectorListView.Init(
                    "_ContactManager",
                    "UserSelectorListView_ContactManager",
                    true,
                    false,
                    "",
                    new Array(ASC.CRM.Resources.CurrentUser),
                    null,
                    new Array(Teamlab.profile.id),
                    "#wtlfAccessRights");

            jq.tmpl("makePublicPanelTemplate",
                    {
                        Title: ASC.CRM.Resources.CRMContactResource.MakePublicPanelTitleForImportContacts,
                        Description: ASC.CRM.Resources.CRMContactResource.MakePublicPanelDescrForContact,
                        IsPublicItem: false,
                        CheckBoxLabel: ASC.CRM.Resources.CRMContactResource.MakePublicPanelCheckBoxLabelForImportContacts
                    }).appendTo("#wtlfMakePublicPanel");
            //isPublic


            ASC.CRM.UserSelectorListView.Init(
                                "_Notify",
                                "UserSelectorListView_Notify",
                                true,
                                false,
                                "",
                                [],
                                null,
                                [],
                                "#userSelectorListViewContainer");

            if (!(typeof window.columnSelectorData === "undefined")) {
                for (var i = 0, n = window.columnSelectorData.length; i < n; i++) {
                    window.columnSelectorData[i].title = Encoder.htmlDecode(window.columnSelectorData[i].title);
                }

                _renderTable(window.columnSelectorData, jq("#tblFieldList tbody"));
            }

            jq("#privateSettingsBlock").parent().css("padding", "0px");
            var text = jq("#privateSettingsBlock").parent().find("span:first").text() + ":";
            jq("#privateSettingsBlock").parent().find("span:first").replaceWith(
                jq("<div><div>").text(text).css("font-weight", "bold")
            );

            _initOtherActionMenu();
        },

        changeContactType: function() {
            jq("#tblFieldList tbody").html("");

            jq(window.columnSelectorData).each(function() {
                this.disabled = false;
            });

            _renderTable(window.columnSelectorData, jq("#tblFieldList tbody"));
        },

        changeWebFormKey: function() {

            if (!confirm(ASC.CRM.Resources.CRMJSResource.ConfirmChangeKey + "\n" + ASC.CRM.Resources.CRMJSResource.ConfirmChangeKeyNote))
            { return false; }

            AjaxPro.WebToLeadFormView.ChangeWebFormKey(
               function(res) {
                   if (res.error != null) {
                       alert(res.error.Message);
                   }
                   jq("#properties_webFormKey input").val(res.value);
                   jq("#webFormKeyContainer").html(res.value);
               }
            );
        },

        generateSampleForm: function() {
            if (!_validateInputData()) { return; }

            var fieldListInfo = jQuery.map(jq("#tblFieldList input:checked"),
                   function(a) {
                       return jq(a).data("fieldInfo");
                   });
            var tagListInfo = jq.map(jq("#tagContainer .tag_title"),
                   function(item) {
                       return {
                           name: "tag_" + jq(item).text().trim(),
                           title: jq(item).text().trim()
                       };
                   });

            if (fieldListInfo.length == 0) {
                jq("#resultContainer, #previewHeader").hide();
                jq("#previewHeader div.content").html("");
                return;
            }

            var notifyList = [],
                managersList = new Array(window.SelectedUsers_ContactManager.CurrentUserID);

            for (var i = 0, n = window.SelectedUsers_ContactManager.IDs.length; i < n; i++) {
                managersList.push(window.SelectedUsers_ContactManager.IDs[i]);
            }
            managersList = managersList.join(",");

            if (window.SelectedUsers_Notify.IDs.length != 0) {
                for (var i = 0, n = window.SelectedUsers_Notify.IDs.length; i < n; i++) {
                    notifyList.push(window.SelectedUsers_Notify.IDs[i]);
                }
                notifyList = notifyList.join(",");
            } else {
                notifyList = "";
            }

            var formContainer = jq("<div>");

            jq.tmpl("sampleFormTmpl", {
                webtoleadfromhandlerPath: ASC.CRM.SettingsPage.WebToLeadFormView.webtoleadfromhandlerPath,
                fieldListInfo: fieldListInfo,
                tagListInfo: tagListInfo,
                returnURL: jq("#returnURL").val(),
                webFormKey: jq("#properties_webFormKey input").val(),
                notifyList: notifyList,
                managersList: managersList,
                isShared: jq("#isPublic").is(":checked"),
                isCompany: jq("input[name=radio]:checked").val() == "company"
            }).appendTo(formContainer);

            var sampleFrom = formContainer.html().replace(/\s{1,}/g, " ");

            jq("#resultContainer textarea").val(sampleFrom);
            jq("#resultContainer").show();
            jq("#previewHeader div.content").html(sampleFrom);
            jq("#previewHeader").show();

        }
    };
})();


ASC.CRM.TaskTemplateView = (function() {

    var _initOtherActionMenu = function() {
        ASC.CRM.Common.removeExportButtons();
        jq("#menuCreateNewTask").bind("click", function() { ASC.CRM.TaskActionView.showTaskPanel(0, "", 0, null, {}); });
    };

    function checkTemplateConatainers() {
        if (jq("#templateConatainerContent li").length != 0) {
            jq("#emptyContent").hide();
            jq("#templateConatainerContent").show();
        } else {
            jq("#templateConatainerContent").hide();
            jq("#emptyContent").show();
        }
    };

    return {
        init: function() {
            if (typeof (window.templateConatainerList) != "undefined") {
                window.templateConatainerList = jq.parseJSON(jQuery.base64.decode(window.templateConatainerList)).response;
            } else {
                window.templateConatainerList = [];
            }

            jq.forceIntegerOnly("#templatePanel #tbxTemplateDisplacement");

            jq.tmpl("templateContainerRow", window.templateConatainerList).appendTo("#templateConatainerContent");
            checkTemplateConatainers();

            _initOtherActionMenu();

            for (var i = 0, n = window.templateConatainerList.length; i < n; i++) {
                if (window.templateConatainerList[i].items) {
                    for (var j = 0, m = window.templateConatainerList[i].items.length; j < m; j++) {
                        ASC.CRM.Common.tooltip("#templateTitle_" + window.templateConatainerList[i].items[j].id, "tooltip");
                    }
                }
            }
        },

        showTemplateConatainerPanel: function(id) {
            if (!id) {
                ASC.CRM.TaskTemplateView.initTemplateConatainerPanel();
                PopupKeyUpActionProvider.EnableEsc = false;
                StudioBlockUIManager.blockUI("#templateConatainerPanel", 500, 500, 0);
            } else {
                Teamlab.getCrmEntityTaskTemplateContainer({}, id, {
                    success: function(params, templateContainer) {
                        ASC.CRM.TaskTemplateView.initTemplateConatainerPanel(templateContainer);
                        PopupKeyUpActionProvider.EnableEsc = false;
                        StudioBlockUIManager.blockUI("#templateConatainerPanel", 500, 500, 0);
                    }
                });
            }
        },

        createTemplateConatainer: function() {
            var title = jq("#templateConatainerTitle").val().trim(),
                view = jq.getURLParam("view"),
                entityType = view != null && view != ""
                            ? view
                            : "contact";

            if (title == "") {
                ShowRequiredError(jq("#templateConatainerTitle"), true);
                return false;
            } else {
                RemoveRequiredErrorClass(jq("#templateConatainerTitle"));
            }

            Teamlab.addCrmEntityTaskTemplateContainer({}, { entityType: entityType, title: title }, {
                success: function(params, templateContainer) {
                    jq.tmpl("templateContainerRow", templateContainer).appendTo("#templateConatainerContent");
                    checkTemplateConatainers();
                },
                before: function(params) {
                    jq("#templateConatainerPanel div.action_block").hide();
                    jq("#templateConatainerPanel div.ajax_info_block").show();
                },
                after: function(params) {
                    jq("#templateConatainerPanel div.ajax_info_block").hide();
                    jq("#templateConatainerPanel div.action_block").show();
                    PopupKeyUpActionProvider.EnableEsc = true;
                    jq.unblockUI();
                }
            });
        },

        editTemplateConatainer: function(containerid) {
            var title = jq("#templateConatainerTitle").val().trim(),
                view = jq.getURLParam("view"),
                entityType = view != null && view != "" ? view : "contact";

            if (title == "") {
                ShowRequiredError(jq("#templateConatainerTitle"), true);
                return false;
            }
            else RemoveRequiredErrorClass(jq("#templateConatainerTitle"));

            Teamlab.updateCrmEntityTaskTemplateContainer({}, containerid, { entityType: entityType, title: title }, {
                success: function(params, templateContainer) {
                    jq("#templateContainerBody_" + containerid).remove();
                    var newTemlateContainer = jq.tmpl("templateContainerRow", templateContainer);
                    jq("#templateContainerHeader_" + containerid).replaceWith(newTemlateContainer);
                },
                before: function(params) {
                    jq("#templateConatainerPanel div.action_block").hide();
                    jq("#templateConatainerPanel div.ajax_info_block").show();
                },
                after: function(params) {
                    jq("#templateConatainerPanel div.ajax_info_block").hide();
                    jq("#templateConatainerPanel div.action_block").show();
                    PopupKeyUpActionProvider.EnableEsc = true;
                    jq.unblockUI();
                }
            });
        },

        deleteTemplateConatainer: function(containerid) {
            Teamlab.removeCrmEntityTaskTemplateContainer({}, containerid, {
                success: function(params, templateContainer) {
                    jq("#templateContainerHeader_" + containerid).remove();
                    jq("#templateContainerBody_" + containerid).remove();
                    checkTemplateConatainers();
                },
                before: function(params) {
                    jq("#templateContainerHeader_" + containerid + " label.crm-addBtn").hide();
                    jq("#templateContainerHeader_" + containerid + " label.crm-editLink").hide();
                    jq("#templateContainerHeader_" + containerid + " label.crm-deleteBtn").hide();
                    jq("#templateContainerHeader_" + containerid + " img.loaderImg").show();
                }
            });
        },

        initTemplateConatainerPanel: function(container) {
            if (!container) {
                jq("#templateConatainerTitle").val("");
                RemoveRequiredErrorClass(jq("#templateConatainerTitle"));
                jq("#templateConatainerPanel div.containerHeaderBlock td:first")
                        .text(ASC.CRM.TaskTemplateView.ConatainerPanel_AddHeaderText);
                jq("#templateConatainerPanel div.action_block a.button.blue")
                        .text(ASC.CRM.TaskTemplateView.ConatainerPanel_AddButtonText)
                        .unbind("click").click(function() {
                            ASC.CRM.TaskTemplateView.createTemplateConatainer();
                        });
            } else {
                jq("#templateConatainerTitle").val(container.title);
                RemoveRequiredErrorClass(jq("#templateConatainerTitle"));
                jq("#templateConatainerPanel div.containerHeaderBlock td:first")
                        .text(jq.format(ASC.CRM.TaskTemplateView.ConatainerPanel_EditHeaderText, container.title));
                jq("#templateConatainerPanel div.action_block a.button.blue")
                        .text(ASC.CRM.Resources.CRMJSResource.SaveChanges)
                        .unbind("click").click(function() {
                            ASC.CRM.TaskTemplateView.editTemplateConatainer(container.id);
                        });
            }
        },

        toggleCollapceExpand: function(elem) {
            $Obj = jq(elem);

            if ($Obj.hasClass("headerCollapse")) {
                var body = $Obj.parents("li").next();
                if (body.find("table").length > 0) {
                    body.show();
                }
            } else {
                $Obj.parents("li").next().hide();
            }
            $Obj.toggleClass("headerExpand");
            $Obj.toggleClass("headerCollapse");
        },

        showTemplatePanel: function(containerid, id) {
            if (!id) {
                ASC.CRM.TaskTemplateView.initTemplatePanel(containerid);
                PopupKeyUpActionProvider.EnableEsc = false;
                StudioBlockUIManager.blockUI("#templatePanel", 500, 500, 0);
            } else {
                Teamlab.getCrmEntityTaskTemplate({}, id, {
                    success: function(params, template) {
                        ASC.CRM.TaskTemplateView.initTemplatePanel(containerid, template);
                        PopupKeyUpActionProvider.EnableEsc = false;
                        StudioBlockUIManager.blockUI("#templatePanel", 500, 500, 0);
                    }
                });
            }
        },

        createTemplate: function(containerid) {
            var data = ASC.CRM.TaskTemplateView.getCurrentTemplateData();
            data.containerid = containerid;

            if (data.title == "") {
                ShowRequiredError(jq("#tbxTemplateTitle"), true);
                return false;
            } else {
                RemoveRequiredErrorClass(jq("#tbxTemplateTitle"));
            }

            if (data.responsibleid == null || data.responsibleid == "") {
                if (jq.browser.mobile === true) {
                    RemoveRequiredErrorClass(jq("#taskTemplateViewAdvUsrSrContainer select:first"), true);
                } else {
                    RemoveRequiredErrorClass(jq("#templatePanel .inputUserName:first"), true);
                }
                return false;
            } else {
                if (jq.browser.mobile === true) {
                    RemoveRequiredErrorClass(jq("#taskTemplateViewAdvUsrSrContainer select:first"));
                } else {
                    RemoveRequiredErrorClass(jq("#templatePanel .inputUserName:first"));
                }
            }

            Teamlab.addCrmEntityTaskTemplate({}, data, {
                success: function(params, template) {
                    jq.tmpl("templateRow", template).appendTo("#templateContainerBody_" + containerid);
                    if (jq("#templateContainerBody_" + containerid + " table").length > 0) {
                        jq("#templateContainerBody_" + containerid).show();
                    }
                    ASC.CRM.Common.tooltip("#templateTitle_" + template.id, "tooltip");
                },
                before: function(params) {
                    jq("#templatePanel div.action_block").hide();
                    jq("#templatePanel div.ajax_info_block").show();
                },
                after: function(params) {
                    jq("#templatePanel div.ajax_info_block").hide();
                    jq("#templatePanel div.action_block").show();
                    PopupKeyUpActionProvider.EnableEsc = true;
                    jq.unblockUI();
                }
            });
        },

        editTemplate: function(containerid, templateid) {
            var data = ASC.CRM.TaskTemplateView.getCurrentTemplateData();
            data.containerid = containerid,
            data.id = templateid;

            if (data.title == "") {
                ShowRequiredError(jq("#tbxTemplateTitle"), true);
                return false;
            } else {
                RemoveRequiredErrorClass(jq("#tbxTemplateTitle"));
            }

            if (data.responsibleid == null) {
                if (jq.browser.mobile === true) {
                    ShowRequiredError(jq("#taskTemplateViewAdvUsrSrContainer select:first"), true);
                } else {
                    ShowRequiredError(jq("#templatePanel .inputUserName:first"), true);
                }
                return false;
            } else {
                if (jq.browser.mobile === true) {
                    RemoveRequiredErrorClass(jq("#taskTemplateViewAdvUsrSrContainer select:first"));
                } else {
                    RemoveRequiredErrorClass(jq("#templatePanel .inputUserName:first"));
                }
            }

            Teamlab.updateCrmEntityTaskTemplate({}, data, {
                success: function(params, template) {
                    var newTemlate = jq.tmpl("templateRow", template);
                    jq("#templateRow_" + templateid).replaceWith(newTemlate);
                    ASC.CRM.Common.tooltip("#templateTitle_" + template.id, "tooltip");
                },
                before: function(params) {
                    jq("#templatePanel div.action_block").hide();
                    jq("#templatePanel div.ajax_info_block").show();
                },
                after: function(params) {
                    jq("#templatePanel div.ajax_info_block").hide();
                    jq("#templatePanel div.action_block").show();
                    PopupKeyUpActionProvider.EnableEsc = true;
                    jq.unblockUI();
                }
            });
        },

        deleteTemplate: function(templateid) {
            Teamlab.removeCrmEntityTaskTemplate({}, templateid, {
                success: function(params, template) {
                    jq("#templateRow_" + templateid).remove();
                    if (jq("#templateContainerBody_" + template.containerID + " table").length == 0)
                        jq("#templateContainerBody_" + template.containerID).hide();
                },
                before: function(params) {
                    jq("#templateRow_" + templateid + " label.crm-addBtn").hide();
                    jq("#templateRow_" + templateid + " label.crm-editLink").hide();
                    jq("#templateRow_" + templateid + " label.crm-deleteBtn").hide();
                    jq("#templateRow_" + templateid + " img.loaderImg").show();
                }
            });
        },

        initTemplatePanel: function(containerid, template) {
            if (!template) {
                jq("#templatePanel div.containerHeaderBlock td:first")
                        .text(ASC.CRM.TaskTemplateView.TemplatePanel_AddHeaderText);
                jq("#templatePanel div.action_block a.button.blue")
                        .text(ASC.CRM.TaskTemplateView.TemplatePanel_AddButtonText)
                        .unbind("click").click(function() {
                            ASC.CRM.TaskTemplateView.createTemplate(containerid);
                        });
                jq("#tbxTemplateTitle").val("");
                jq("#tbxTemplateDescribe").val("");
                ASC.CRM.TaskTemplateView.setTemplateDeadlineFromTicks();
                jq("#notifyResponsible").removeAttr("checked");
                window.taskTemplateResponsibleSelector.ClearFilter();
                window.taskTemplateResponsibleSelector.ChangeDepartment(window.taskTemplateResponsibleSelector.Groups[0].ID);
                var obj = window.taskTemplateCategorySelector.getRowByContactID(0);
                window.taskTemplateCategorySelector.changeContact(obj);

            } else {
                jq("#templatePanel div.containerHeaderBlock td:first")
                        .text(jq.format(ASC.CRM.TaskTemplateView.TemplatePanel_EditHeaderText, template.title));
                jq("#templatePanel div.action_block a.button.blue")
                        .text(ASC.CRM.Resources.CRMJSResource.SaveChanges).unbind("click").click(function() {
                            ASC.CRM.TaskTemplateView.editTemplate(containerid, template.id);
                        });
                jq("#tbxTemplateTitle").val(template.title);
                jq("#tbxTemplateDescribe").val(template.description);
                ASC.CRM.TaskTemplateView.setTemplateDeadlineFromTicks(template.offsetTicks, template.deadLineIsFixed);
                jq("#notifyResponsible").prop("checked", template.isNotify);
                window.taskTemplateResponsibleSelector.ClearFilter();
                window.taskTemplateResponsibleSelector.ChangeDepartment(window.taskTemplateResponsibleSelector.Groups[0].ID);

                var obj;
                if (!jq.browser.mobile) {
                    obj = document.getElementById("User_" + template.responsible.id);
                    if (obj != null)
                        window.taskTemplateResponsibleSelector.SelectUser(obj);
                } else {
                    obj = jq("#taskResponsibleSelector select option[value=" + template.responsible.id + "]");
                    if (obj.length > 0) {
                        window.taskTemplateResponsibleSelector.SelectUser(obj);
                        jq(obj).prop("selected", true);
                    }
                }

                obj = window.taskTemplateCategorySelector.getRowByContactID(template.category.id);
                window.taskTemplateCategorySelector.changeContact(obj);
            }
        },

        getTemplateDeadlineTicks: function() {
            var displacement = jq("#tbxTemplateDisplacement").val().trim() != ""
                    ? parseInt(jq("#tbxTemplateDisplacement").val())
                    : 0,
                hour = displacement * 24 + parseInt(jq("#templateDeadlineHours option:selected").val()),
                minute = parseInt(jq("#templateDeadlineMinutes option:selected").val());

            return ((hour * 3600) + minute * 60) * 10000000;
        },

        setTemplateDeadlineFromTicks: function(ticks, isFixed) {
            if (!ticks && !isFixed) {
                jq("#deadLine_fixed").prop("checked", true);
                jq("#tbxTemplateDisplacement").val("0");
                jq("#templateDeadlineHours option:first").prop("selected", true);
                jq("#templateDeadlineMinutes option:first").prop("selected", true);
            } else {
                if (isFixed) {
                    jq("#deadLine_fixed").prop("checked", true);
                } else {
                    jq("#deadLine_not_fixed").prop("checked", true);
                }

                var minute = ((ticks / 10000000) % 3600) / 60,
                    hour = (parseInt((ticks / 10000000) / 3600)) % 24,
                    displacement = parseInt((parseInt((ticks / 10000000) / 3600)) / 24);

                jq("#tbxTemplateDisplacement").val(displacement);
                jq("#optDeadlineHours_" + hour).prop("selected", true);
                jq("#optDeadlineMinutes_" + minute).prop("selected", true);
            }
        },

        getCurrentTemplateData: function() {
            var data = {
                title: jq("#tbxTemplateTitle").val().trim(),
                description: jq("#tbxTemplateDescribe").val().trim(),
                responsibleid: window.taskTemplateResponsibleSelector.SelectedUserId,
                categoryid: window.taskTemplateCategorySelector.CategoryID,
                isNotify: jq("#notifyResponsible").is(":checked"),
                offsetTicks: ASC.CRM.TaskTemplateView.getTemplateDeadlineTicks(),
                deadLineIsFixed: jq('#templatePanel #deadLine_fixed').is(":checked")
            };
            return data;
        },

        getDeadlineDisplacement: function(ticks, isFixed) {
            var minute = ((ticks / 10000000) % 3600) / 60,
                hour = (parseInt((ticks / 10000000) / 3600)) % 24,
                displacement = parseInt((parseInt((ticks / 10000000) / 3600)) / 24);

            if (isFixed) {
                return jq.format(ASC.CRM.Resources.CRMJSResource.TemplateFixedDeadline, displacement, hour, minute);
            } else {
                return jq.format(ASC.CRM.Resources.CRMJSResource.TemplateNotFixedDeadline, displacement, hour, minute);
            }
        }
    };
})();